@@ -11,6 +11,9 @@
 
 
 #include <iostream>
+#include <string>
+
+#include <cstring>
 
 #include <sys/stat.h>
 
@@ -18,23 +21,34 @@
 
 #include <gtk/gtk.h>
 #include <gtk/gtknotebook.h>
+#include <gdk/gdkkeysyms.h>
+
 #include "gp_cppsupport/printoutputselector.h"
 #include "pp_cms.h"
 #include "pp_units.h"
 #include "pp_scaling.h"
 #include "stpui_widgets/stpui_optionbook.h"
 #include "splashscreen/splashscreen.h"
+
 #include "miscwidgets/patheditor.h"
-#include "support/pathsupport.h"
 #include "miscwidgets/imageselector.h"
 #include "miscwidgets/generaldialogs.h"
+#include "miscwidgets/pixbufview.h"
+#include "miscwidgets/simplecombo.h"
+
+#include "support/debug.h"
+#include "support/pathsupport.h"
 #include "support/rangeparser.h"
 #include "support/progressbar.h"
+#include "support/util.h"
+
 #include "imageutils/tiffsave.h"
 #include "imageutils/jpegsave.h"
+#include "imagesource/pixbuf_from_imagesource.h"
+
 #include "profilemanager/profileselector.h"
 #include "profilemanager/intentselector.h"
-#include "support/util.h"
+
 #include "config.h"
 #include "gettext.h"
 #define _(x) gettext(x)
@@ -46,17 +60,167 @@ using namespace std;
 
 // Colour Management
 
-struct colourmanagementdata
-{
-	PhotoPrint_State *state;
-	GtkWidget *dialog;
-};
-
 void ColourManagement_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 {
 	pp_cms_run_dialog(&state,parent);
 }
 
+
+class crt_dialog : public Progress
+{
+	public:
+	crt_dialog(GtkWindow *parent,PhotoPrint_State &state)
+	: Progress(), state(state), storedlabel(NULL), resultlabel(NULL), progressbar(NULL), button(NULL), image(NULL), existing(NULL)
+	{
+		window=gtk_dialog_new_with_buttons(_("Colour Response Hash"),
+			parent,GtkDialogFlags(0),
+			GTK_STOCK_OK,GTK_RESPONSE_OK,
+			NULL);
+		gtk_window_set_default_size(GTK_WINDOW(window),400,50);
+		gtk_widget_show(window);
+
+		GtkWidget *vbox=GTK_DIALOG(window)->vbox;
+		GtkWidget *hbox=gtk_hbox_new(FALSE,0);
+		gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,8);
+		gtk_widget_show(hbox);
+
+		GtkWidget *table=gtk_table_new(2,6,FALSE);
+		gtk_table_set_col_spacing(GTK_TABLE(table),0,12);
+		gtk_table_set_col_spacing(GTK_TABLE(table),1,12);
+		gtk_table_set_row_spacings(GTK_TABLE(table),12);
+		gtk_box_pack_start(GTK_BOX(hbox),table,TRUE,TRUE,12);
+		gtk_widget_show(table);
+
+
+		GtkWidget *tmp;
+		tmp=gtk_label_new(_("This \"hash\" value is derived from the printer driver's output,\n"
+							"and can be used to verify that no changes have occurred which\n"
+							"might invalidate a colour profile for the printer"));
+		gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,3,0,1);
+		gtk_widget_show(tmp);
+
+
+		image=gtk_image_new_from_stock(GTK_STOCK_DIALOG_QUESTION,GTK_ICON_SIZE_DIALOG);
+		gtk_table_attach_defaults(GTK_TABLE(table),image,2,3,1,3);
+		gtk_widget_show(image);
+
+				
+		tmp=gtk_label_new(_("Current hash:"));
+		gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,1,1,2);
+		gtk_widget_show(tmp);
+
+		tmp=gtk_label_new(_("Stored in preset:"));
+		gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,1,2,3);
+		gtk_widget_show(tmp);
+
+		existing=state.printoutput.FindString("ResponseHash");
+		if(strlen(existing)==0)
+			storedlabel=gtk_label_new(_("No hash stored yet"));
+		else
+			storedlabel=gtk_label_new(existing);
+
+		gtk_label_set_selectable(GTK_LABEL(storedlabel),TRUE);
+		gtk_table_attach_defaults(GTK_TABLE(table),storedlabel,1,2,2,3);
+		gtk_widget_show(storedlabel);
+
+
+		resultlabel=gtk_label_new(_("Calculating hash of printer response\nwith current driver settings"));
+		gtk_table_attach_defaults(GTK_TABLE(table),resultlabel,1,3,4,7);
+		gtk_widget_show(resultlabel);
+
+
+		button=gtk_button_new_with_label(_("Store in preset"));
+		g_signal_connect(G_OBJECT(button),"clicked",G_CALLBACK(clicked),this);
+		gtk_table_attach_defaults(GTK_TABLE(table),button,0,1,5,6);
+		gtk_widget_set_sensitive(button,FALSE);
+		gtk_widget_show(button);
+
+
+		progressbar=gtk_progress_bar_new();
+		gtk_table_attach_defaults(GTK_TABLE(table),progressbar,1,2,1,2);
+		gtk_widget_show(progressbar);
+
+		responsetag=state.printer.GetResponseHash(this);
+
+		gtk_widget_destroy(progressbar);
+		progressbar=NULL;
+
+		tmp=gtk_label_new(responsetag.c_str());
+		gtk_label_set_selectable(GTK_LABEL(tmp),TRUE);
+		gtk_table_attach_defaults(GTK_TABLE(table),tmp,1,2,1,2);
+		gtk_widget_show(tmp);
+
+
+		if(strcmp(existing,responsetag.c_str())==0)
+		{
+			gtk_label_set_text(GTK_LABEL(resultlabel),_("MATCH - No changes in driver\nor printer settings detected"));
+			gtk_image_set_from_stock(GTK_IMAGE(image),GTK_STOCK_YES,GTK_ICON_SIZE_DIALOG);
+		}
+		else
+		{
+			if(strlen(existing)==0)
+				gtk_label_set_text(GTK_LABEL(resultlabel),_("This hash can be used to detect changes\nin printer settings or driver versions\nthat might invalidate colour profiles"));
+			else
+				gtk_label_set_text(GTK_LABEL(resultlabel),_("MISMATCH - change detected in printer response!\nPrinter's colour profile might no longer be accurate."));
+			gtk_widget_set_sensitive(button,TRUE);
+			gtk_image_set_from_stock(GTK_IMAGE(image),GTK_STOCK_DIALOG_WARNING,GTK_ICON_SIZE_DIALOG);
+		}
+
+		gtk_dialog_run(GTK_DIALOG(window));
+	}
+	~crt_dialog()
+	{
+		gtk_widget_destroy(window);
+	}
+	static void clicked(GtkWidget *wid,gpointer userdata)
+	{
+		crt_dialog *c=(crt_dialog *)userdata;
+		c->state.printoutput.SetString("ResponseHash",c->responsetag.c_str());
+		c->existing=c->responsetag.c_str();
+		gtk_label_set_text(GTK_LABEL(c->storedlabel),c->existing);
+	}
+	bool DoProgress(int i,int maxi)
+	{
+		if(!progressbar)
+			return(false);
+		Progress::DoProgress(i,maxi);
+		if(maxi)
+		{
+			float v=i;
+			v/=maxi;
+			
+			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progressbar),v);
+		}
+		else
+			gtk_progress_bar_pulse(GTK_PROGRESS_BAR(progressbar));
+
+		while(gtk_events_pending())
+			gtk_main_iteration_do(false);
+		return(true);
+	}
+	void SetMessage(const char *msg)
+	{
+	}
+	protected:
+	PhotoPrint_State &state;
+	GtkWidget *window;
+	GtkWidget *storedlabel;
+	GtkWidget *resultlabel;
+	GtkWidget *progressbar;
+	GtkWidget *button;
+	GtkWidget *image;
+	const char *existing;
+	string responsetag;
+};
+
+
+void ColourResponseTag_Dialog(GtkWidget *parent,PhotoPrint_State &state)
+{
+	crt_dialog dlg(GTK_WINDOW(parent),state);
+}
+
+
+
 void Units_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 {
 	enum Units units=state.GetUnits();
@@ -90,17 +254,17 @@ struct printsetupdata
 
 static void driver_changed(GtkWidget *wid,gpointer data)
 {
-	cerr << "In driver_changed()" << endl;
+	Debug[TRACE] << "In driver_changed()" << endl;
 
 	struct printsetupdata *cbd=(struct printsetupdata *)data;
 	PrintOutput *po=&cbd->state->printoutput;
 
 	const char *driver=po->FindString("Driver");
-	cerr << "Driver=" << driver << endl;
+	Debug[TRACE] << "Driver=" << driver << endl;
 
 	if(!(cbd->state->printer.SetDriver(po->FindString("Driver"))))
 	{
-		cerr << "Setting driver failed - reverting to default" << endl;
+		Debug[TRACE] << "Setting driver failed - reverting to default" << endl;
 		po->SetString("Driver",DEFAULT_PRINTER_DRIVER);
 		cbd->state->printer.SetDriver(po->FindString("Driver"));
 		printoutputselector_refresh(PRINTOUTPUTSELECTOR(cbd->printoutput));
@@ -114,7 +278,7 @@ void PrintSetup_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 {
 	printsetupdata dialogdata;
 
-	cerr << "Opening print setup dialog" << endl;
+	Debug[TRACE] << "Opening print setup dialog" << endl;
 	
 	dialogdata.state=&state;
 	dialogdata.backupvars=stp_vars_create_copy(state.printer.stpvars);
@@ -127,12 +291,12 @@ void PrintSetup_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 		NULL);
 	gtk_window_set_default_size(GTK_WINDOW(dialogdata.dialog),500,350);
 
-	cerr << "Creating PrintOutput widget..." << endl;
+	Debug[TRACE] << "Creating PrintOutput widget..." << endl;
 
 	dialogdata.printoutput=printoutputselector_new(&state.printoutput);
 	g_object_ref(G_OBJECT(dialogdata.printoutput));
 
-	cerr << "Created PrintOutput widget..." << endl;
+	Debug[TRACE] << "Created PrintOutput widget..." << endl;
 
 	dialogdata.custompage.name=_("Output");
 	dialogdata.custompage.widget=dialogdata.printoutput;
@@ -182,11 +346,12 @@ void About_Dialog(GtkWindow *parent)
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),image,FALSE,FALSE,0);
 	gtk_widget_show(image);
 	
-	GtkWidget *label=gtk_label_new("PhotoPrint "
+	GtkWidget *label=gtk_label_new(_("PhotoPrint "
 	PACKAGE_VERSION
 	" - copyright (c) 2004-2009\n"
 	"by Alastair M. Robinson (amr@blackfiveservices.co.uk)\nDistributed under the terms\nof the GNU General Public Licence.\n"
-	"See the file 'COPYING' for more details.");
+	"See the file 'COPYING' for more details.\n"
+	"The borders pack is released under the Creative Commons\nAttribution 2.0 UK: England & Wales Licence"));
 	gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_CENTER);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),label,TRUE,FALSE,5);
 	gtk_widget_show(label);
@@ -269,7 +434,7 @@ void Paths_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 	gtk_box_pack_start(GTK_BOX(hbox),borderpath,TRUE,TRUE,0);
 	gtk_widget_show(borderpath);
 
-	GtkWidget *imagesel=imageselector_new(&dd.sp,false);
+	GtkWidget *imagesel=imageselector_new(&dd.sp,GTK_SELECTION_NONE);
 	dd.is=IMAGESELECTOR(imagesel);
 	g_signal_connect(imagesel,"changed",G_CALLBACK(paths_changed),&dd);
 	gtk_box_pack_start(GTK_BOX(hbox),imagesel,FALSE,FALSE,0);
@@ -300,7 +465,7 @@ void Paths_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 	gtk_box_pack_start(GTK_BOX(hbox),bgpath,TRUE,TRUE,0);
 	gtk_widget_show(bgpath);
 
-	imagesel=imageselector_new(&bgdd.sp,false);
+	imagesel=imageselector_new(&bgdd.sp,GTK_SELECTION_NONE);
 	bgdd.is=IMAGESELECTOR(imagesel);
 	g_signal_connect(imagesel,"changed",G_CALLBACK(paths_changed),&bgdd);
 	gtk_box_pack_start(GTK_BOX(hbox),imagesel,FALSE,FALSE,0);
@@ -343,7 +508,7 @@ char *ImageMask_Dialog(GtkWindow *parent,PhotoPrint_State &state,char *oldfn)
 		NULL);
 	gtk_window_set_default_size(GTK_WINDOW(dialog),250,350);
 
-	GtkWidget *imagesel=imageselector_new(&state.bordersearchpath,true);
+	GtkWidget *imagesel=imageselector_new(&state.bordersearchpath,GTK_SELECTION_SINGLE);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),imagesel,TRUE,TRUE,0);
 	gtk_widget_show(imagesel);
 
@@ -503,7 +668,7 @@ void ExportTiff_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 							GTK_MESSAGE_WARNING,GTK_BUTTONS_OK_CANCEL,
 							"File exists - OK to overwrite?");
 							gint response=gtk_dialog_run (GTK_DIALOG (confirm));
-							cerr << "Response: " << response << endl;
+							Debug[TRACE] << "Response: " << response << endl;
 							if(response!=GTK_RESPONSE_OK)
 							{
 								g_free(outputfilename);
@@ -521,27 +686,27 @@ void ExportTiff_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 						if(profileactive)
 							state.profilemanager.SetString("ExportProfile",profile);
 		
-						cerr << "Page range: " << pagerange << endl;
-						cerr << "Resolution: " << res << endl;
-						cerr << "Using profile?: " << profileactive << endl;
+						Debug[TRACE] << "Page range: " << pagerange << endl;
+						Debug[TRACE] << "Resolution: " << res << endl;
+						Debug[TRACE] << "Using profile?: " << profileactive << endl;
 						if(profileactive)
-						cerr << "  " << profile << endl;
+						Debug[TRACE] << "  " << profile << endl;
 		
 						RangeParser rp(pagerange,state.layout->GetPages());
 						int p;
 						CMTransformFactory *factory=state.profilemanager.GetTransformFactory();
 						while((p=rp.Next()))
 						{
-							cerr << "Exporting page " << p << " of " <<  state.layout->GetPages() << endl;
-							cerr << "To filename... ";
+							Debug[TRACE] << "Exporting page " << p << " of " <<  state.layout->GetPages() << endl;
+							Debug[TRACE] << "To filename... ";
 							char *ftmp;
 							if(state.layout->GetPages()>1)
 								ftmp=SerialiseFilename(outputfilename,p,state.layout->GetPages());
 							else
 								ftmp=strdup(outputfilename);
-							cerr << ftmp << endl;
+							Debug[TRACE] << ftmp << endl;
 
-							ImageSource *is=state.layout->GetImageSource(p-1,CM_COLOURDEVICE_EXPORT,factory,res);
+							ImageSource *is=state.layout->GetImageSource(p-1,CM_COLOURDEVICE_EXPORT,factory,res,true);
 							if(is)
 							{
 								ProgressBar p(_("Exporting..."),true,GTK_WIDGET(parent));
@@ -549,9 +714,9 @@ void ExportTiff_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 								if(profileactive)
 								{
 									CMSProfile *prof=state.profilemanager.GetProfile(CM_COLOURDEVICE_EXPORT);
-									cerr << "Got profile " << prof->GetFilename() << endl;
+									Debug[TRACE] << "Got profile " << prof->GetFilename() << endl;
 									is->SetEmbeddedProfile(prof);
-									cerr << "Set profile - saving..." << endl;
+									Debug[TRACE] << "Set profile - saving..." << endl;
 								}
 
 								TIFFSaver ts(ftmp,is,save16bit);
@@ -708,27 +873,27 @@ void ExportJPEG_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 						if(profileactive)
 							state.profilemanager.SetString("ExportProfile",profile);
 		
-						cerr << "Page range: " << pagerange << endl;
-						cerr << "Resolution: " << res << endl;
-						cerr << "Using profile?: " << profileactive << endl;
+						Debug[TRACE] << "Page range: " << pagerange << endl;
+						Debug[TRACE] << "Resolution: " << res << endl;
+						Debug[TRACE] << "Using profile?: " << profileactive << endl;
 						if(profileactive)
-						cerr << "  " << profile << endl;
+						Debug[TRACE] << "  " << profile << endl;
 		
 						RangeParser rp(pagerange,state.layout->GetPages());
 						int p;
 						CMTransformFactory *factory=state.profilemanager.GetTransformFactory();
 						while((p=rp.Next()))
 						{
-							cerr << "Exporting page " << p << " of " <<  state.layout->GetPages() << endl;
-							cerr << "To filename... ";
+							Debug[TRACE] << "Exporting page " << p << " of " <<  state.layout->GetPages() << endl;
+							Debug[TRACE] << "To filename... ";
 							char *ftmp;
 							if(state.layout->GetPages()>1)
 								ftmp=SerialiseFilename(outputfilename,p,state.layout->GetPages());
 							else
 								ftmp=strdup(outputfilename);
-							cerr << ftmp << endl;
+							Debug[TRACE] << ftmp << endl;
 
-							ImageSource *is=state.layout->GetImageSource(p-1,CM_COLOURDEVICE_EXPORT,factory,res);
+							ImageSource *is=state.layout->GetImageSource(p-1,CM_COLOURDEVICE_EXPORT,factory,res,true);
 							if(is)
 							{
 								ProgressBar p(_("Exporting..."),true,GTK_WIDGET(parent));
@@ -736,9 +901,9 @@ void ExportJPEG_Dialog(GtkWindow *parent,PhotoPrint_State &state)
 								if(profileactive)
 								{
 									CMSProfile *prof=state.profilemanager.GetProfile(CM_COLOURDEVICE_EXPORT);
-									cerr << "Got profile " << prof->GetFilename() << endl;
+									Debug[TRACE] << "Got profile " << prof->GetFilename() << endl;
 									is->SetEmbeddedProfile(prof);
-									cerr << "Set profile - saving..." << endl;
+									Debug[TRACE] << "Set profile - saving..." << endl;
 								}
 
 								JPEGSaver ts(ftmp,is,quality);
@@ -777,7 +942,7 @@ char *Background_Dialog(GtkWindow *parent,PhotoPrint_State &state,char *oldfn)
 		NULL);
 	gtk_window_set_default_size(GTK_WINDOW(dialog),250,350);
 
-	GtkWidget *imagesel=imageselector_new(&state.backgroundsearchpath,true);
+	GtkWidget *imagesel=imageselector_new(&state.backgroundsearchpath,GTK_SELECTION_SINGLE);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),imagesel,TRUE,TRUE,0);
 	gtk_widget_show(imagesel);
 
@@ -826,9 +991,9 @@ static void refreshprofiledialog(GtkWidget *wid,gpointer obj)
 	if(profile)
 		profile=st->state->profilemanager.SearchPaths(profile);
 
-	cerr << "Got profileactive: " << profileactive << endl;
+	Debug[TRACE] << "Got profileactive: " << profileactive << endl;
 	if(profile)
-		cerr << "Got profile: " << profile << endl;
+		Debug[TRACE] << "Got profile: " << profile << endl;
 
 	if(profileactive)
 		st->ii->AssignProfile(profile);
@@ -838,14 +1003,14 @@ static void refreshprofiledialog(GtkWidget *wid,gpointer obj)
 	LCMSWrapper_Intent intent=intentselector_getintent(INTENTSELECTOR(st->intent));
 	st->ii->SetRenderingIntent(intent);
 
-	cerr << "Rendering intent: " << intent << endl;
+	Debug[TRACE] << "Rendering intent: " << intent << endl;
 
-	cerr << "Assigned" << endl;
+	Debug[TRACE] << "Assigned" << endl;
 
 	// FIXME - deal with Intent here.
 
 	GdkPixbuf *tn=st->ii->GetThumbnail();
-	cerr << "Drawing new image" << endl;
+	Debug[TRACE] << "Drawing new image" << endl;
 	gtk_image_set_from_pixbuf(GTK_IMAGE(st->preview),tn);
 }
 
@@ -1006,3 +1171,5 @@ void SetCustomProfileDialog(GtkWindow *parent,PhotoPrint_State &state,Layout_Ima
 		free(oldcustomprofile);
 	gtk_widget_destroy(dialog);
 }
+
+
