@@ -13,6 +13,8 @@
 
 #include <stdlib.h>
 
+#include "../support/debug.h"
+
 #include "imagesource_rotate.h"
 
 using namespace std;
@@ -92,7 +94,15 @@ ISDataType *ImageSource_Rotate::GetRow(int row)
 							}
 							break;
 						default:
-							throw "Rotate: Only 1, 3 or 4 samples per pixel are currently supported";
+							for(x=firstrow;x<lastrow;++x)
+							{
+								int sx=(source->width-1)-x;
+								for(int s=0;s<samplesperpixel;++s)
+								{
+									c=src[sx*samplesperpixel+s];
+									dst[(x-firstrow)*samplesperrow+s]=c;
+								}
+							}
 							break;
 					}
 					if(TestBreak())
@@ -155,7 +165,15 @@ ISDataType *ImageSource_Rotate::GetRow(int row)
 							}
 							break;
 						default:
-							throw "Rotate: Only 1, 3 or 4 samples per pixel are currently supported";
+							for(x=firstrow;x<lastrow;++x)
+							{
+								int sx=(width-1)-x;
+								for(int s=0;s<samplesperpixel;++s)
+								{
+									c=src[sx*samplesperpixel+s];
+									dst[x*samplesperrow+s]=c;
+								}
+							}
 							break;
 					}
 					if(TestBreak())
@@ -165,53 +183,6 @@ ISDataType *ImageSource_Rotate::GetRow(int row)
 			row-=spanfirstrow;
 			return(spanbuffer+row*samplesperrow);		
 			break;
-			
-#if 0
-			src=source->GetRow((height-1)-row);
-			dst=rowbuffer;
-
-			switch(samplesperpixel)
-			{
-				case 1:
-					for(x=0;x<width;++x)
-					{
-						int sx=(width-1)-x;
-						c=src[sx];
-						dst[x]=c;
-					}
-					break;
-				case 3:
-					for(x=0;x<width;++x)
-					{
-						int sx=(width-1)-x;
-						c=src[sx*3];
-						dst[x*3]=c;
-						c=src[sx*3+1];
-						dst[x*3+1]=c;
-						c=src[sx*3+2];
-						dst[x*3+2]=c;
-					}
-					break;
-				case 4:
-					for(x=0;x<width;++x)
-					{
-						int sx=(width-1)-x;
-						c=src[sx*4];
-						dst[x*4]=c;
-						c=src[sx*4+1];
-						dst[x*4+1]=c;
-						c=src[sx*4+2];
-						dst[x*4+2]=c;
-						c=src[sx*4+3];
-						dst[x*4+3]=c;
-					}
-					break;
-				default:
-					throw "Rotate: Only 1, 3 or 4 samples per pixel are currently supported";
-					break;
-			}
-#endif
-			break;
 		case 270:
 			if((row<spanfirstrow) || (row>=(spanfirstrow+spanrows)))
 			{
@@ -263,7 +234,15 @@ ISDataType *ImageSource_Rotate::GetRow(int row)
 							}
 							break;
 						default:
-							throw "Rotate: Only 1, 3 or 4 samples per pixel are currently supported";
+							for(x=firstrow;x<lastrow;++x)
+							{
+								int sx=x;
+								for(int s=0;s<samplesperpixel;++s)
+								{
+									c=src[sx*samplesperpixel+s];
+									dst[(x-firstrow)*samplesperrow+s]=c;
+								}
+							}
 							break;
 					}
 					if(TestBreak())
@@ -305,14 +284,14 @@ ImageSource_Rotate::ImageSource_Rotate(ImageSource *source,int rotation,int span
 	{
 		case 0:
 		case 180:
-			cerr << "Rotate: caching entire image for 180 degree rotation" << endl;
+			Debug[COMMENT] << "Rotate: caching entire image for 180 degree rotation" << endl;
 			this->spanrows=source->height+1;
 			break;
 		case 90:
 		case 270:
 			if(!source->randomaccess)
 			{
-				cerr << "Rotate: source doesn't support random access - caching entire image" << endl;
+				Debug[COMMENT] << "Rotate: source doesn't support random access - caching entire image" << endl;
 				this->spanrows=source->width+1;
 			}
 			break;
@@ -321,7 +300,7 @@ ImageSource_Rotate::ImageSource_Rotate(ImageSource *source,int rotation,int span
 	spanfirstrow=-this->spanrows-1;
 	samplesperrow=width*samplesperpixel;
 
-	cerr << "Span buffer will be " << this->spanrows << " high" << endl;
+	Debug[TRACE] << "Span buffer will be " << this->spanrows << " high" << endl;
 
 	if((rotation==90) || (rotation==270) || (rotation==180))
 		spanbuffer=(ISDataType *)malloc(this->spanrows*(sizeof(ISDataType)*samplesperrow));
