@@ -31,6 +31,8 @@
 #include "imagesource/imagesource_promote.h"
 
 #include "photoprint_state.h"
+
+#include "support/debug.h"
 #include "support/progress.h"
 #include "support/util.h"
 #include "support/layoutrectangle.h"
@@ -49,9 +51,60 @@ ConfigTemplate LayoutDB::Template[]=
 };
 
 
+LayoutIterator::LayoutIterator(Layout &header) : header(header)
+{
+}
+
+LayoutIterator::~LayoutIterator()
+{
+}
+
+Layout_ImageInfo *LayoutIterator::FirstImage()
+{
+	iterator=header.imagelist.begin();
+	if(iterator==header.imagelist.end())
+		return(NULL);
+	return(*iterator);
+}
+
+Layout_ImageInfo *LayoutIterator::NextImage()
+{
+	if(iterator==header.imagelist.end())
+		return(NULL);
+	++iterator;
+	if(iterator==header.imagelist.end())
+		return(NULL);
+	return(*iterator);
+}
+
+Layout_ImageInfo *LayoutIterator::FirstSelected()
+{
+	Layout_ImageInfo *ii=FirstImage();
+	while(ii)
+	{
+		if(ii->GetSelected())
+			return(ii);
+		ii=NextSelected();
+	}
+	return(NULL);
+}
+
+Layout_ImageInfo *LayoutIterator::NextSelected()
+{
+	Layout_ImageInfo *ii=NextImage();
+	while(ii)
+	{
+		if(ii->GetSelected())
+			return(ii);
+		ii=NextSelected();
+	}
+	return(NULL);
+}
+
+
 Layout::Layout(PhotoPrint_State &state,Layout *oldlayout)
-	: PageExtent(), xoffset(0), yoffset(0), pages(1), currentpage(0), backgroundfilename(NULL), background(NULL),
-	backgroundtransformed(NULL), state(state), imagelist(NULL), iterator(NULL), factory(NULL), gc(NULL)
+	: PageExtent(), state(state), xoffset(0), yoffset(0), pages(1), currentpage(0), backgroundfilename(NULL), background(NULL),
+	backgroundtransformed(NULL), imagelist(NULL), factory(NULL), gc(NULL)
 {
 	factory=state.profilemanager.GetTransformFactory();
 }
@@ -59,16 +112,14 @@ Layout::Layout(PhotoPrint_State &state,Layout *oldlayout)
 
 Layout::~Layout()
 {
-	if(imagelist)
+	LayoutIterator it(*this);
+	Layout_ImageInfo *ii=it.FirstImage();
+	while(ii)
 	{
-		for(unsigned int i=0;i<g_list_length(imagelist);++i)
-		{
-			GList *element=g_list_nth(imagelist,i);
-			Layout_ImageInfo *ii=(Layout_ImageInfo *)element->data;
-			delete ii;
-		}
+		delete ii;
+		ii=it.FirstImage();
 	}
-	g_list_free(imagelist);
+
 	if(gc)
 		g_object_unref(G_OBJECT(gc));
 	if(factory)
@@ -91,17 +142,14 @@ void Layout::Clear()
 {
 	SetBackground(NULL);
 	FlushHRPreviews();
-	if(imagelist)
+
+	LayoutIterator it(*this);
+	Layout_ImageInfo *ii=it.FirstImage();
+	while(ii)
 	{
-		for(unsigned int i=0;i<g_list_length(imagelist);++i)
-		{
-			GList *element=g_list_nth(imagelist,i);
-			Layout_ImageInfo *ii=(Layout_ImageInfo *)element->data;
-			delete ii;
-		}
+		delete ii;
+		ii=it.NextImage();
 	}
-	g_list_free(imagelist);
-	imagelist=NULL;
 	pages=1;
 }
 
@@ -109,7 +157,7 @@ void Layout::Clear()
 // DUMMY FUNCTION - should be overridden by subclasses
 int Layout::AddImage(const char *filename,bool allowcropping,PP_ROTATION rotation)
 {
-	cerr << "AddImage: Dummy function - should be overridden" << endl;
+	Debug[ERROR] << "AddImage: Dummy function - should be overridden" << endl;
 	return(0);
 }
 
@@ -117,13 +165,13 @@ int Layout::AddImage(const char *filename,bool allowcropping,PP_ROTATION rotatio
 // DUMMY FUNCTION - should be overridden by subclasses
 void Layout::CopyImage(Layout_ImageInfo *ii)
 {
-	cerr << "CopyImage: Dummy function - should be overridden" << endl;
+	Debug[ERROR] << "CopyImage: Dummy function - should be overridden" << endl;
 }
 
 
-ImageSource *Layout::GetImageSource(int page,CMColourDevice target,CMTransformFactory *factory,int res)
+ImageSource *Layout::GetImageSource(int page,CMColourDevice target,CMTransformFactory *factory,int res,bool completepage)
 {
-	cerr << "GetImageSource: Dummy function - should be overridden" << endl;
+	Debug[ERROR] << "GetImageSource: Dummy function - should be overridden" << endl;
 	return(NULL);
 }
 
@@ -145,7 +193,7 @@ IS_TYPE Layout::GetColourSpace(CMColourDevice target)
 		else if(strcmp(cs,"CMYK")==0)
 			colourspace=IS_TYPE_CMYK;
 		else
-			cerr << "PrintColourSpace is set to an unknown colour space!" << endl;
+			Debug[WARN] << "PrintColourSpace is set to an unknown colour space!" << endl;
 	}
 	return(colourspace);
 }
@@ -153,27 +201,23 @@ IS_TYPE Layout::GetColourSpace(CMColourDevice target)
 
 void Layout::TransferImages(Layout *oldlayout,Progress *p)
 {
-	if(currentpage>=pages);
 	if(oldlayout)
 	{
-		int count=0,img=0;
-		Layout_ImageInfo *ii=oldlayout->FirstImage();
+		int count=oldlayout->imagelist.size();
+
+		LayoutIterator it(*oldlayout);
+		Layout_ImageInfo *ii=it.FirstImage();
+		int img=0;
 		while(ii)
 		{
-			++count;
-			ii=oldlayout->NextImage();
-		}
-		ii=oldlayout->FirstImage();
-		while(ii)
-		{
-			++img;
 			CopyImage(ii);
-			ii=oldlayout->NextImage();
+			ii=it.NextImage();
 			if(p)
 			{
 				if(!(p->DoProgress(img,count)))
 					ii=NULL;
 			}
+			++img;
 		}
 	}
 	if(currentpage>=pages)
@@ -252,58 +296,90 @@ void Layout::Print(Progress *p)
 
 void (*Layout::SetUnitFunc())(GtkWidget *wid,enum Units unit)
 {
-	cerr << "This function should be overridden" << endl;
+	Debug[ERROR] << "This function should be overridden" << endl;
 	return(NULL);
 }
 
 
+void Layout::MakeGC(GtkWidget *widget)
+{
+	bgcol.pixel=0;
+	bgcol.red=65535;
+	bgcol.green=65535;
+	bgcol.blue=65535;
+
+	bgcol2.pixel=0;
+	bgcol2.red=32767;
+	bgcol2.green=32767;
+	bgcol2.blue=32767;
+
+	// Use printer and monitor profiles to calculate "paper white".
+	// If there's no display profile, then we can use the Default RGB profile instead...
+//		Debug[TRACE] << "Checking for Display Profile..." << endl;
+	CMSProfile *targetprof;
+	CMColourDevice target=CM_COLOURDEVICE_NONE;
+	if((targetprof=state.profilemanager.GetProfile(CM_COLOURDEVICE_PRINTERPROOF)))
+		target=CM_COLOURDEVICE_PRINTERPROOF;
+	else if((targetprof=state.profilemanager.GetProfile(CM_COLOURDEVICE_DISPLAY)))
+		target=CM_COLOURDEVICE_DISPLAY;
+	else if((targetprof=state.profilemanager.GetProfile(CM_COLOURDEVICE_DEFAULTRGB)))
+		target=CM_COLOURDEVICE_DEFAULTRGB;
+	if(targetprof)
+		delete targetprof;
+
+
+	if(target!=CM_COLOURDEVICE_NONE)
+	{
+		CMSTransform *transform=NULL;
+//			Debug[TRACE] << "Creating default->monitor transform..." << endl;
+		transform = factory->GetTransform(target,IS_TYPE_RGB,LCMSWRAPPER_INTENT_DEFAULT);
+		if(transform)
+		{
+//				Debug[TRACE] << "Applying transform..." << endl;
+
+			ISDataType rgbtriple[3];
+			rgbtriple[0]=bgcol.red;
+			rgbtriple[1]=bgcol.green;
+			rgbtriple[2]=bgcol.blue;
+			transform->Transform(rgbtriple,rgbtriple,1);
+			bgcol.red=rgbtriple[0];
+			bgcol.green=rgbtriple[1];
+			bgcol.blue=rgbtriple[2];
+
+			rgbtriple[0]=bgcol2.red;
+			rgbtriple[1]=bgcol2.green;
+			rgbtriple[2]=bgcol2.blue;
+			transform->Transform(rgbtriple,rgbtriple,1);
+			bgcol2.red=rgbtriple[0];
+			bgcol2.green=rgbtriple[1];
+			bgcol2.blue=rgbtriple[2];
+		}
+	}
+	gc=gdk_gc_new(widget->window);
+}
+
+
+void Layout::DrawPreviewBorder(GtkWidget *widget)
+{
+	if(!gc)
+		MakeGC(widget);
+
+	gdk_gc_set_rgb_fg_color(gc,&bgcol2);
+	gdk_draw_rectangle (widget->window,
+		gc,TRUE,
+		0,0,
+		widget->allocation.width,widget->allocation.height);
+}
+
+
 void Layout::DrawPreviewBG(GtkWidget *widget,int xpos,int ypos,int width,int height)
 {
 	if(!gc)
-	{
-		bgcol.pixel=0;
-		bgcol.red=65535;
-		bgcol.green=65535;
-		bgcol.blue=65535;
+		MakeGC(widget);
 
-		// Use printer and monitor profiles to calculate "paper white".
-		// If there's no display profile, then we can use the Default RGB profile instead...
-//		cerr << "Checking for Display Profile..." << endl;
-		CMSProfile *targetprof;
-		CMColourDevice target=CM_COLOURDEVICE_NONE;
-		if((targetprof=state.profilemanager.GetProfile(CM_COLOURDEVICE_PRINTERPROOF)))
-			target=CM_COLOURDEVICE_PRINTERPROOF;
-		else if((targetprof=state.profilemanager.GetProfile(CM_COLOURDEVICE_DISPLAY)))
-			target=CM_COLOURDEVICE_DISPLAY;
-		else if((targetprof=state.profilemanager.GetProfile(CM_COLOURDEVICE_DEFAULTRGB)))
-			target=CM_COLOURDEVICE_DEFAULTRGB;
-		if(targetprof)
-			delete targetprof;
+	DrawPreviewBorder(widget);
 
-
-		if(target!=CM_COLOURDEVICE_NONE)
-		{
-			CMSTransform *transform=NULL;
-//			cerr << "Creating default->monitor transform..." << endl;
-			transform = factory->GetTransform(target,IS_TYPE_RGB,LCMSWRAPPER_INTENT_DEFAULT);
-			if(transform)
-			{
-//				cerr << "Applying transform..." << endl;
-
-				ISDataType rgbtriple[3];
-				rgbtriple[0]=bgcol.red;
-				rgbtriple[1]=bgcol.green;
-				rgbtriple[2]=bgcol.blue;
-				transform->Transform(rgbtriple,rgbtriple,1);
-				bgcol.red=rgbtriple[0];
-				bgcol.green=rgbtriple[1];
-				bgcol.blue=rgbtriple[2];
-			}
-		}
-
-		gc=gdk_gc_new(widget->window);
-		gdk_gc_set_rgb_fg_color(gc,&bgcol);
-	}
+	gdk_gc_set_rgb_fg_color(gc,&bgcol);
 	gdk_draw_rectangle (widget->window,
 		gc,TRUE,
 		xpos,ypos,
@@ -317,9 +393,9 @@ void Layout::DrawPreviewBG(GtkWidget *widget,int xpos,int ypos,int width,int hei
 	scale/=pagewidth;
 	sr.Scale(scale);
 
-	gdk_draw_rectangle (widget->window,
-		widget->style->mid_gc[widget->state],FALSE,
-		xpos+sr.x,ypos+sr.y,sr.w,sr.h);
+//	gdk_draw_rectangle (widget->window,
+//		widget->style->mid_gc[widget->state],FALSE,
+//		xpos+sr.x,ypos+sr.y,sr.w,sr.h);
 
 	if(background)
 	{
@@ -360,78 +436,34 @@ void Layout::DrawPreview(GtkWidget *widget,int xpos,int ypos,int width,int heigh
 {
 	DrawPreviewBG(widget,xpos,ypos,width,height);
 
-	Layout_ImageInfo *ii=FirstImage();
+	LayoutIterator it(*this);
+	Layout_ImageInfo *ii=it.FirstImage();
 
 	while(ii)
 	{
 		if(currentpage==ii->page)
 			ii->DrawThumbnail(widget,xpos,ypos,width,height);
-		ii=NextImage();
+		ii=it.NextImage();
 	}
 }
 
 
-Layout_ImageInfo *Layout::FirstImage()
-{
-	iterator=imagelist;
-	if(iterator)
-		return((Layout_ImageInfo *)iterator->data);
-	else
-		return(NULL);
-}
-
-
-Layout_ImageInfo *Layout::NextImage()
-{
-	if(iterator)
-		iterator=g_list_next(iterator);
-	if(iterator)
-		return((Layout_ImageInfo *)iterator->data);
-	else
-		return(NULL);
-}
-
-
 int Layout::GetPages()
 {
 	return(pages);
 }
 
 
-Layout_ImageInfo *Layout::FirstSelected()
-{
-	Layout_ImageInfo *ii=FirstImage();
-	while(ii)
-	{
-		if(ii->GetSelected())
-			return(ii);
-		ii=NextSelected();
-	}
-	return(NULL);
-}
-
-
-Layout_ImageInfo *Layout::NextSelected()
-{
-	Layout_ImageInfo *ii=NextImage();
-	while(ii)
-	{
-		if(ii->GetSelected())
-			return(ii);
-		ii=NextSelected();
-	}
-	return(NULL);
-}
-
-
 int Layout::CountSelected()
 {
 	int count=0;
-	Layout_ImageInfo *ii=FirstSelected();
+
+	LayoutIterator it(*this);
+	Layout_ImageInfo *ii=it.FirstSelected();
 	while(ii)
 	{
 		++count;
-		ii=NextSelected();
+		ii=it.NextSelected();
 	}
 	return(count);
 }
@@ -439,22 +471,24 @@ int Layout::CountSelected()
 
 void Layout::SelectAll()
 {
-	Layout_ImageInfo *ii=FirstImage();
+	LayoutIterator it(*this);
+	Layout_ImageInfo *ii=it.FirstImage();
 	while(ii)
 	{
 		ii->SetSelected(true);
-		ii=NextImage();
+		ii=it.NextImage();
 	}
 }
 
 
 void Layout::SelectNone()
 {
-	Layout_ImageInfo *ii=FirstSelected();
+	LayoutIterator it(*this);
+	Layout_ImageInfo *ii=it.FirstSelected();
 	while(ii)
 	{
 		ii->SetSelected(false);
-		ii=NextSelected();
+		ii=it.NextSelected();
 	}
 }
 
@@ -485,18 +519,18 @@ void Layout::SetBackground(const char *filename)
 		free(backgroundfilename);
 	backgroundfilename=NULL;
 	
-//	cerr << "Setting background..." << endl;
+//	Debug[TRACE] << "Setting background..." << endl;
 	if(filename && strlen(filename)>0)
 	{
 		GError *err=NULL;
 
 		backgroundfilename=strdup(filename);
 
-//		cerr << "Attempting to load background from: " << backgroundfilename << endl;
+//		Debug[TRACE] << "Attempting to load background from: " << backgroundfilename << endl;
 		background=egg_pixbuf_get_thumbnail_for_file (backgroundfilename, EGG_PIXBUF_THUMBNAIL_LARGE, &err);
 		if(!background)
 		{
-//			cerr << "Failed." << endl;
+//			Debug[TRACE] << "Failed." << endl;
 			try
 			{
 				ImageSource *src=ISLoadImage(backgroundfilename);
@@ -517,14 +551,14 @@ void Layout::SetBackground(const char *filename)
 			}
 			catch(const char *err)
 			{
-				cerr << "Error: " << err << endl;
+				Debug[ERROR] << "Error: " << err << endl;
 			}	
 			if(!background)
 			{
 				if(err && err->message)
-					cerr << "Error: " << err->message << endl;
+					Debug[ERROR] << "Error: " << err->message << endl;
 				else
-					cerr << "Can't get mask thumbnail" << endl;
+					Debug[ERROR] << "Can't get mask thumbnail" << endl;
 				free(backgroundfilename);
 				backgroundfilename=NULL;
 			}
@@ -535,19 +569,20 @@ void Layout::SetBackground(const char *filename)
 
 void Layout::FlushThumbnails()
 {
-	Layout_ImageInfo *ii=FirstImage();
+	LayoutIterator it(*this);
+	Layout_ImageInfo *ii=it.FirstImage();
 	while(ii)
 	{
 		ii->FlushThumbnail();
-		ii=NextImage();	
+		ii=it.NextImage();	
 	}
 
 	// We obtain the mutices to ensure the rendering threads have completed.
-	ii=FirstImage();
+	ii=it.FirstImage();
 	while(ii)
 	{
 		ii->ObtainMutex();
-		ii=NextImage();
+		ii=it.NextImage();
 	}
 
 	// Can't delete this until after the thumbnails have been flushed,
@@ -565,11 +600,11 @@ void Layout::FlushThumbnails()
 	gc=NULL;
 
 	// And now we release the mutices.
-	ii=FirstImage();
+	ii=it.FirstImage();
 	while(ii)
 	{
 		ii->ReleaseMutex();
-		ii=NextImage();
+		ii=it.NextImage();
 	}
 }
 
@@ -581,21 +616,26 @@ void Layout::FlushHRPreviews()
 	// to exit - then wait for them in turn while we flush
 	// the previews.  Quicker than signal / wait / signal / wait / etc.
 	CancelRenderThreads();
-	Layout_ImageInfo *ii=FirstImage();
+
+	LayoutIterator it(*this);
+	Layout_ImageInfo *ii=it.FirstImage();
 	while(ii)
 	{
 		ii->FlushHRPreview();
-		ii=NextImage();	
+		ii=it.NextImage();	
 	}
 }
 
 
 void Layout::CancelRenderThreads()
 {
-	Layout_ImageInfo *ii=FirstImage();
+	LayoutIterator it(*this);
+	Layout_ImageInfo *ii=it.FirstImage();
 	while(ii)
 	{
 		ii->CancelRenderThread();
-		ii=NextImage();	
+		ii=it.NextImage();	
 	}
 }
+
+
