@@ -1,6 +1,7 @@
 #ifndef LAYOUT_H
 #define LAYOUT_H
 
+#include <list>
 #include <stdio.h>
 #include <glib.h>
 #include <gtk/gtkwidget.h>
@@ -30,6 +31,21 @@ class Progress;
 #define PPLAYOUT_DUPLICATE 64
 
 
+class LayoutIterator
+{
+	public:
+	LayoutIterator(Layout &header);
+	~LayoutIterator();
+	Layout_ImageInfo *FirstImage();
+	Layout_ImageInfo *NextImage();
+	Layout_ImageInfo *FirstSelected();
+	Layout_ImageInfo *NextSelected();
+	protected:
+	Layout &header;
+	std::list<Layout_ImageInfo *>::iterator iterator;
+};
+
+
 class Layout : public virtual PageExtent
 {
 	public:
@@ -48,7 +64,8 @@ class Layout : public virtual PageExtent
 	virtual int FreeSlots();	// Count the number of free slots on the current page
 	virtual void Reflow();
 
-	virtual ImageSource *GetImageSource(int page,CMColourDevice target=CM_COLOURDEVICE_PRINTER,CMTransformFactory *factory=NULL,int res=0);
+	virtual ImageSource *GetImageSource(int page,CMColourDevice target=CM_COLOURDEVICE_PRINTER,
+		CMTransformFactory *factory=NULL,int res=0,bool completepage=false);
 	virtual IS_TYPE GetColourSpace(CMColourDevice target);	// Do we still need this?
 	virtual void UpdatePageSize();
 	virtual void LayoutToDB(LayoutDB &db);
@@ -56,10 +73,6 @@ class Layout : public virtual PageExtent
 
 	// Image list / selections
 
-	virtual Layout_ImageInfo *FirstImage();
-	virtual Layout_ImageInfo *NextImage();
-	virtual Layout_ImageInfo *FirstSelected();
-	virtual Layout_ImageInfo *NextSelected();
 	virtual int CountSelected();
 	virtual void SelectAll();
 	virtual void SelectNone();
@@ -67,12 +80,12 @@ class Layout : public virtual PageExtent
 	virtual int GetCurrentPage();
 	virtual void SetCurrentPage(int page);
 
-
 	// UI-related
 
 	virtual GtkWidget *CreateWidget();
 	virtual void RefreshWidget(GtkWidget *widget);
 	virtual void Print(Progress *p);
+	virtual void DrawPreviewBorder(GtkWidget *widget);
 	virtual void DrawPreviewBG(GtkWidget *widget,int xpos,int ypos,int width,int height);
 	virtual void DrawPreview(GtkWidget *widget,int xpos,int ypos,int width,int height);
 	virtual void SetBackground(const char *filename);
@@ -84,7 +97,7 @@ class Layout : public virtual PageExtent
 	PhotoPrint_State &state;
 
 	protected:
-
+	void MakeGC(GtkWidget *widget);
 	// Xoffset and yoffset are the top left corner of the print.
 	// Some layouts will want to set this to the top/left margin
 	// The best way to do so is to override the print method.
@@ -95,15 +108,20 @@ class Layout : public virtual PageExtent
 	char *backgroundfilename;
 	GdkPixbuf *background;
 	GdkPixbuf *backgroundtransformed;
+
 	// Housekeeping
-	GList *imagelist;
-	GList *iterator;
+//	GList *imagelist;
+//	GList *iterator;
+	list<Layout_ImageInfo *> imagelist;
+
 	// For thumbnails and preview...
 	CMTransformFactory *factory;
 	GdkGC *gc;
 	GdkColor bgcol;
+	GdkColor bgcol2;
 	friend class Layout_ImageInfo;
 	friend class hr_payload;
+	friend class LayoutIterator;
 };
 
 
