@@ -32,6 +32,9 @@
 #include "imagesource/imagesource_invert.h"
 
 #include "photoprint_state.h"
+
+#include "support/debug.h"
+
 #include "support/progress.h"
 #include "support/util.h"
 #include "support/layoutrectangle.h"
@@ -41,10 +44,37 @@
 using namespace std;
 
 
+class PPIS_Histogram : public ImageSource
+{
+	public:
+	PPIS_Histogram(ImageSource *source,PPHistogram &hist) : ImageSource(source), source(source,hist), histogram(hist)
+	{
+		Debug[TRACE] << "PPIS_Histogram obtaining Histogram mutex in exclusive mode from " << long(Thread::GetThreadID()) << endl;
+		histogram.ObtainMutex();
+		Debug[TRACE] << "Histogram address: " << long(&histogram) << endl;
+	}
+	virtual ~PPIS_Histogram()
+	{
+		Debug[TRACE] << "PPIS_Histogram triggering complete signal" << endl;
+		histogram.Trigger();
+		Debug[TRACE] << "PPIS_Histogram releasing Histogram mutex from " << long(Thread::GetThreadID()) << endl;
+		histogram.ReleaseMutex();
+	}
+	virtual ISDataType *GetRow(int row)
+	{
+		return(source.GetRow(row));
+	}
+	protected:
+	ImageSource_Histogram source;
+	PPHistogram &histogram;
+};
+
+
 Layout_ImageInfo::Layout_ImageInfo(Layout &layout, const char *filename, int page, bool allowcropping, PP_ROTATION rotation)
 	: PPEffectHeader(), page(page), allowcropping(allowcropping), crop_hpan(CENTRE), crop_vpan(CENTRE),
 	rotation(rotation), layout(layout), maskfilename(NULL), thumbnail(NULL), mask(NULL), hrpreview(NULL),
-	selected(false), customprofile(NULL), customintent(LCMSWRAPPER_INTENT_DEFAULT), hrrenderthread(NULL)
+	selected(false), customprofile(NULL), customintent(LCMSWRAPPER_INTENT_DEFAULT), hrrenderthread(NULL),
+	threadevents(), histogram(threadevents)
 {
 	bool relative=true;
 
@@ -60,6 +90,8 @@ Layout_ImageInfo::Layout_ImageInfo(Layout &layout, const char *filename, int pag
 		this->filename=strdup(filename);
 
 	ImageSource *is=ISLoadImage(this->filename);
+	if(!is)
+		throw "Can't open image!";
 	width=is->width;
 	height=is->height;
 	xres=is->xres;
@@ -71,10 +103,11 @@ Layout_ImageInfo::Layout_ImageInfo(Layout &layout, const char *filename, int pag
 }
 
 
-Layout_ImageInfo::Layout_ImageInfo(Layout &layout, Layout_ImageInfo *ii, int page, bool allowcropping, PP_ROTATION rotation)
-	: PPEffectHeader(*ii), page(page), allowcropping(allowcropping), crop_hpan(CENTRE), crop_vpan(CENTRE),
-	rotation(rotation), layout(layout), maskfilename(NULL), thumbnail(NULL), mask(NULL), hrpreview(NULL),
-	selected(false), customprofile(NULL), customintent(LCMSWRAPPER_INTENT_DEFAULT), hrrenderthread(NULL)
+Layout_ImageInfo::Layout_ImageInfo(Layout &layout, Layout_ImageInfo *ii, int page)
+	: PPEffectHeader(*ii), page(page), allowcropping(false), crop_hpan(CENTRE), crop_vpan(CENTRE),
+	rotation(PP_ROTATION_AUTO), layout(layout), maskfilename(NULL), thumbnail(NULL), mask(NULL), hrpreview(NULL),
+	selected(false), customprofile(NULL), customintent(LCMSWRAPPER_INTENT_DEFAULT), hrrenderthread(NULL),
+	threadevents(), histogram(threadevents)
 {
 	// Effects are copied by the "PPEffectHeader(*ii) above
 	if(ii)
@@ -91,7 +124,7 @@ Layout_ImageInfo::Layout_ImageInfo(Layout &layout, Layout_ImageInfo *ii, int pag
 		if(ii->customprofile)
 		{
 			customprofile=strdup(ii->customprofile);
-			cerr << "Copying profile: " << ii->customprofile << endl;
+			Debug[TRACE] << "Copying profile: " << ii->customprofile << endl;
 		}
 		customintent=ii->customintent;
 	
@@ -122,11 +155,13 @@ Layout_ImageInfo::~Layout_ImageInfo()
 		g_object_unref(thumbnail);
 	if(mask)
 		g_object_unref(mask);
-	layout.imagelist=g_list_remove(layout.imagelist,this);
+
+	layout.imagelist.remove(this);
+
 	if(customprofile)
 		free(customprofile);
 	free(filename);
-	cerr << "Layout_ImageInfo successfully disposed" << endl;
+	Debug[COMMENT] << "Layout_ImageInfo successfully disposed" << endl;
 }
 
 
@@ -135,43 +170,45 @@ Layout_ImageInfo::~Layout_ImageInfo()
 // an image, then defers to the main thread to perform the
 // actual rendering.
 
-class hr_payload
+class hr_payload : public PTMutex, public ThreadFunction
 {
 	public:
 	hr_payload(ProfileManager *p,CMTransformFactory *f,Layout_ImageInfo *ii,GtkWidget *wid,int x,int y,int w,int h)
-		: profman(p), factory(f), ii(ii), widget(wid), xpos(x), ypos(y), width(w), height(h), transformed(NULL), thread(NULL)
+		: PTMutex(), ThreadFunction(), profman(p), factory(f), ii(ii), widget(wid),
+		xpos(x), ypos(y), width(w), height(h), transformed(NULL), thread(this)
 	{
+		thread.Start();
+		thread.WaitSync();
 	}
 	~hr_payload()
 	{
-		if(thread)
-			delete thread;
+	}
+	void Stop()
+	{
+		thread.Stop();
 	}
 	static bool testbreakfunc(void *ud)
 	{
 		Thread *t=(Thread *)ud;
 		return(t->TestBreak());
 	}
-	static int ThreadFunc(Thread *t,void *ud)
+	virtual int Entry(Thread &t)
 	{
-		hr_payload *p=(hr_payload *)ud;
+		ObtainMutex();	// We use this to avoid race conditions when cleaning up.
 
-		p->thread=t;
-
-		cerr << "Subthread - about to obtain mutex" << endl;
-//		p->ii->ObtainMutexShared();
 		// To avoid a deadlock situation if, say, the Apply Profile dialog is open and GTK decides that
 		// now is a good time to redraw the window, we only attempt the mutex, and bail out if it fails.
 		int count=10;
-		while(!p->ii->AttemptMutexShared())
+		while(!ii->AttemptMutexShared())
 		{
 			if(count==0)
 			{
-				cerr << "Giving up attempt on mutex - bailing out" << endl;
+				Debug[WARN] << "Giving up attempt on mutex - bailing out" << endl;
 				// The calling thread is waiting for us to acknowledge startup, so we have to send
 				// the Sync before bailing out.
-				t->SendSync();
-				g_timeout_add(1,hr_payload::CleanupFunc,p);
+				t.SendSync();
+				g_timeout_add(1,hr_payload::CleanupFunc,this);
+				ReleaseMutex();
 				return(0);
 			}
 #ifdef WIN32
@@ -182,7 +219,7 @@ class hr_payload
 			--count;
 		}
 
-		t->SendSync();
+		t.SendSync();
 
 		// We sleep briefly before doing anything time-intensive - that way we can bail out rapidly
 		// if the user's doing something heavily interactive, like panning an image, or resizing the window
@@ -194,20 +231,22 @@ class hr_payload
 #else
 			usleep(10000);
 #endif
-			if(t->TestBreak())
+			if(t.TestBreak())
 			{
-				cerr << "Got break signal while pausing - Releasing" << endl;
-				p->ii->ReleaseMutex();
-				g_timeout_add(1,hr_payload::CleanupFunc,p);
+//				Debug[TRACE] << "Got break signal while pausing - Releasing" << endl;
+				ii->ReleaseMutex();
+				g_timeout_add(1,hr_payload::CleanupFunc,this);
+				ReleaseMutex();
 				return(0);
 			}
 		}
 
-		if(t->TestBreak())
+		if(t.TestBreak())
 		{
-			cerr << "Subthread releasing mutex and cancelling" << endl;
-			p->ii->ReleaseMutex();
-			g_timeout_add(1,hr_payload::CleanupFunc,p);
+//			Debug[TRACE] << "Subthread releasing mutex and cancelling" << endl;
+			ii->ReleaseMutex();
+			g_timeout_add(1,hr_payload::CleanupFunc,this);
+			ReleaseMutex();
 			return(0);
 		}
 
@@ -216,59 +255,59 @@ class hr_payload
 			CMSProfile *targetprof;
 
 			CMColourDevice tdev=CM_COLOURDEVICE_NONE;
-			if((targetprof=p->profman->GetProfile(CM_COLOURDEVICE_PRINTERPROOF)))
+			if((targetprof=profman->GetProfile(CM_COLOURDEVICE_PRINTERPROOF)))
 				tdev=CM_COLOURDEVICE_PRINTERPROOF;
-			else if((targetprof=p->profman->GetProfile(CM_COLOURDEVICE_DISPLAY)))
+			else if((targetprof=profman->GetProfile(CM_COLOURDEVICE_DISPLAY)))
 				tdev=CM_COLOURDEVICE_DISPLAY;
-			else if((targetprof=p->profman->GetProfile(CM_COLOURDEVICE_DEFAULTRGB)))
+			else if((targetprof=profman->GetProfile(CM_COLOURDEVICE_DEFAULTRGB)))
 				tdev=CM_COLOURDEVICE_DEFAULTRGB;
 			if(targetprof)
 				delete targetprof;
 
-			if(t->TestBreak())
+			if(t.TestBreak())
 			{
-				cerr << "Subthread releasing mutex and cancelling" << endl;
-				p->ii->ReleaseMutex();
-				g_timeout_add(1,hr_payload::CleanupFunc,p);
+//				Debug[TRACE] << "Subthread releasing mutex and cancelling" << endl;
+				ii->ReleaseMutex();
+				g_timeout_add(1,hr_payload::CleanupFunc,this);
 				return(0);
 			}
 
-			cerr << "Generating high-res preview - Using tdev: " << tdev << endl;
+//			Debug[TRACE] << "Generating high-res preview - Using tdev: " << tdev << endl;
 
-			ImageSource *is=p->ii->GetImageSource(tdev,p->factory);
+			ImageSource *is=ii->GetImageSource(tdev,factory);
 
-			cerr << "Got imagesource - fitting and rendering" << endl;
+//			Debug[TRACE] << "Got imagesource - fitting and rendering" << endl;
 
 			LayoutRectangle r(is->width,is->height);
-			LayoutRectangle target(p->xpos,p->ypos,p->width,p->height);
+			LayoutRectangle target(xpos,ypos,width,height);
 
-			RectFit *fit=r.Fit(target,p->ii->allowcropping,p->ii->rotation,p->ii->crop_hpan,p->ii->crop_vpan);
+			RectFit *fit=r.Fit(target,ii->allowcropping,ii->rotation,ii->crop_hpan,ii->crop_vpan);
 
 			if(fit->rotation)
 			{
 				ImageSource_Interruptible *ii=new ImageSource_Rotate(is,fit->rotation);
-				ii->SetTestBreak(testbreakfunc,t);
+				ii->SetTestBreak(testbreakfunc,&t);
 				is=ii;
 			}
 			is=ISScaleImageBySize(is,fit->width,fit->height,IS_SCALING_AUTOMATIC);
 			delete fit;
 			// We create new Fit in the idle-function because the hpan/vpan may have changed.
 
-			ProgressThread prog(*t);
-			p->transformed=pixbuf_from_imagesource(is,p->ii->layout.bgcol.red>>8,p->ii->layout.bgcol.green>>8,p->ii->layout.bgcol.blue>>8,&prog);
+			ProgressThread prog(t);
+			transformed=pixbuf_from_imagesource(is,ii->layout.bgcol.red>>8,ii->layout.bgcol.green>>8,ii->layout.bgcol.blue>>8,&prog);
 
 			delete is;
 
-			cerr << "finished - finalising" << endl;
+//			Debug[TRACE] << "finished - finalising" << endl;
 
-			if(p->transformed)
+			if(transformed)
 			{
 				// Now we defer to the main thread...
 				// We add this as a high-priority event because we want it to be
 				// run as soon as possible, and within a gtk_main_iteration() loop
 				// if necessary.
 	//			g_idle_add_full(G_PRIORITY_HIGH,hr_payload::IdleFunc,p,NULL);
-				g_timeout_add(1,hr_payload::IdleFunc,p);
+				g_timeout_add(1,hr_payload::IdleFunc,this);
 
 				// And wait for the main thread to have rendered the preview
 				// Because the rendering will be done via a GTK event callback, there's
@@ -276,26 +315,28 @@ class hr_payload
 				// between the main thread having completed and the idle function being
 				// triggered to draw the thumbnail.  For this reason we'll have to use a
 				// tie-break in the ImageInfo destructor.
-	//			cerr << "Waiting for all-clear from main thread..." << endl;
+	//			Debug[TRACE] << "Waiting for all-clear from main thread..." << endl;
 	//			t->WaitSync();
 			}
 			else
 			{
-				cerr << "Thread cancelled" << endl;
-				g_timeout_add(1,hr_payload::CleanupFunc,p);
-				p->ii->ReleaseMutex();
+//				Debug[TRACE] << "Thread cancelled" << endl;
+				g_timeout_add(1,hr_payload::CleanupFunc,this);
+				ii->ReleaseMutex();
+				ReleaseMutex();
 				return(0);
 			}
 		}
 		catch (const char *err)
 		{
-			cerr << "Subthread caught exception: " << err << endl;
-			g_timeout_add(1,hr_payload::CleanupFunc,p);
+//			Debug[TRACE] << "Subthread caught exception: " << err << endl;
+			g_timeout_add(1,hr_payload::CleanupFunc,this);
 		}
-		cerr << "Subthread waiting for main thread to finish drawing" << endl;
-		p->thread->WaitSync();
-		cerr << "Subthread releasing mutex and exiting" << endl;
-		p->ii->ReleaseMutex();
+		Debug[COMMENT] << "Subthread waiting for main thread to finish drawing" << endl;
+		thread.WaitSync();
+		Debug[COMMENT] << "Subthread releasing mutex and exiting" << endl;
+		ii->ReleaseMutex();
+		ReleaseMutex();
 		return(0);
 	}
 
@@ -303,12 +344,26 @@ class hr_payload
 	static gboolean CleanupFunc(gpointer ud)
 	{
 		hr_payload *p=(hr_payload *)ud;
-		p->thread->SendSync();
+//		Debug[TRACE] << "Main thread sending sync signal" << endl;
+		p->thread.SendSync();
+
+		// There's a race condition here.  Once we send this signal the subthread will
+		// release the mutex - but it's possible this function will have deleted the object first.
+		// To avoid this, we obtain the mutex here, then release it again.
+		// (In actual fact this race condition should be avoided by the fact that
+		// this class's destructor deletes the thread - thus the subthread should be
+		// guaranteed to have exited before this class is deleted.)
+
+//		Debug[TRACE] << "Thread cleanup - race prevention - obtaining mutex from thread " << p->thread.GetThreadID() << endl;
+		p->ObtainMutex();
+//		Debug[TRACE] << "Thread cleanup - race prevention - releasing mutex" << endl;
+		p->ReleaseMutex();
+		Debug[TRACE] << "Done" << endl;
 
 		// We clear the renderthread pointer in the ImageInfo here before deleting it
 		// to avoid the main thread trying to cancel it after deletion.
 		// This should be safe since this function runs on the main thread's context.
-		if(p->ii->hrrenderthread==p->thread)
+		if(p->ii->hrrenderthread==p)
 			p->ii->hrrenderthread=NULL;
 
 		delete p;
@@ -334,9 +389,9 @@ class hr_payload
 		// This idle-function is responsible for disposing of the payload, which
 		// will also delete the thread.
 
-		if(!p->thread->TestBreak())
+		if(!p->thread.TestBreak())
 		{
-			if(p->ii->hrrenderthread==p->thread)
+			if(p->ii->hrrenderthread==p)
 				p->ii->hrrenderthread=NULL;
 
 			p->ii->SetHRPreview(p->transformed);
@@ -357,13 +412,11 @@ class hr_payload
 			
 			if(dw>gdk_pixbuf_get_width(p->transformed))
 			{
-				cerr << "DW too high" << endl;
 				dw=gdk_pixbuf_get_width(p->transformed);
 			}
 
 			if(dh>gdk_pixbuf_get_height(p->transformed))
 			{
-				cerr << "DH too high" << endl;
 				dh=gdk_pixbuf_get_height(p->transformed);
 			}
 
@@ -385,8 +438,27 @@ class hr_payload
 
 			delete fit;
 		}
-		cerr << "Preview drawn - sending sync to sub-thread" << endl;
-		p->thread->SendSync();
+//		Debug[TRACE] << "Preview drawn - sending sync to sub-thread" << endl;
+		p->thread.SendSync();
+
+		// There's a race condition here.  Once we send this signal the subthread will
+		// release the mutex - but it's possible this function will have deleted the object first.
+		// To avoid this, we obtain the mutex here, then release it again.
+		// (In actual fact this race condition should be avoided by the fact that
+		// this class's destructor deletes the thread - thus the subthread should be
+		// guaranteed to have exited before this class is deleted.)
+
+//		Debug[TRACE] << "Thread cleanup - race prevention - obtaining mutex from thread " << p->thread.GetThreadID() << endl;
+		p->ObtainMutex();
+		Debug[COMMENT] << "Thread cleanup - race prevention - releasing mutex" << endl;
+		p->ReleaseMutex();
+//		Debug[TRACE] << "Done" << endl;
+
+		// We clear the renderthread pointer in the ImageInfo here before deleting it
+		// to avoid the main thread trying to cancel it after deletion.
+		// This should be safe since this function runs on the main thread's context.
+		if(p->ii->hrrenderthread==p)
+			p->ii->hrrenderthread=NULL;
 
 		delete p;
 
@@ -400,7 +472,7 @@ class hr_payload
 	int xpos,ypos;
 	int width,height;
 	GdkPixbuf *transformed;
-	Thread *thread;
+	Thread thread;
 };
 
 
@@ -411,28 +483,6 @@ void Layout_ImageInfo::DrawThumbnail(GtkWidget *widget,int xpos,int ypos,int wid
 	RectFit *fit=NULL;
 	int dw,dh;
 
-	// Since the thread (or rather, the idle function it triggers)
-	// is now responsible for its own demise, we no longer have to wait for it.
-	// Furthermore, if there's a thread running, we needn't cancel it here - instead
-	// we can allow it to continue.  Only changes which would lead to flushing the
-	// hi-res preview need to cancel the thread...
-//	cerr << "Cancelling previous thread" << endl;
-//	if(hrrenderthread)
-//		hrrenderthread->Stop();
-//	else
-//		cerr << "No thread" << endl;
-#if 0
-	if(hrrenderthread)
-	{
-		hrrenderthread->Stop();
-		while(!hrrenderthread->TestFinished())
-			gtk_main_iteration();
-		delete hrrenderthread;
-	}
-#endif
-//	hrrenderthread=NULL;
-
-	cerr << "DrawThumbnail - Obtain" << endl;
 	if(thumbnail)
 	{
 		LayoutRectangle r(gdk_pixbuf_get_width(thumbnail),gdk_pixbuf_get_height(thumbnail));
@@ -450,16 +500,11 @@ void Layout_ImageInfo::DrawThumbnail(GtkWidget *widget,int xpos,int ypos,int wid
 			dh=height;
 		
 		if(dw>gdk_pixbuf_get_width(thumbnail))
-		{
-			cerr << "DW too high" << endl;
 			dw=gdk_pixbuf_get_width(thumbnail);
-		}
 
 		if(dh>gdk_pixbuf_get_height(thumbnail))
-		{
-			cerr << "DH too high" << endl;
 			dh=gdk_pixbuf_get_height(thumbnail);
-		}
+
 		if(mask)
 		{
 			transformed=gdk_pixbuf_copy(hrpreview);
@@ -510,16 +555,10 @@ void Layout_ImageInfo::DrawThumbnail(GtkWidget *widget,int xpos,int ypos,int wid
 			dh=height;
 		
 		if(dw>gdk_pixbuf_get_width(transformed))
-		{
-			cerr << "DW too high" << endl;
 			dw=gdk_pixbuf_get_width(transformed);
-		}
 
 		if(dh>gdk_pixbuf_get_height(transformed))
-		{
-			cerr << "DH too high" << endl;
 			dh=gdk_pixbuf_get_height(transformed);
-		}
 
 		if(mask)
 			maskpixbuf(transformed,fit->xoffset,fit->yoffset,dw,dh,mask,
@@ -530,18 +569,12 @@ void Layout_ImageInfo::DrawThumbnail(GtkWidget *widget,int xpos,int ypos,int wid
 		// and if high-res previews are enabled
 		if(hrrenderthread==NULL && layout.state.FindInt("HighresPreviews"))
 		{
-			cerr << "Launching render thread" << endl;
 			if(width>192 && height>192) // Generating lots of thumbs simultaneously is expensive, and if the images are small,
 			{							// highres previews are of limited value anyway.
-				hr_payload *p=new hr_payload(&layout.state.profilemanager,layout.factory,this,widget,xpos,ypos,width,height);
-				hrrenderthread=new Thread(hr_payload::ThreadFunc,p);
-				hrrenderthread->Start();
-				hrrenderthread->WaitSync();
+				hrrenderthread=new hr_payload(&layout.state.profilemanager,layout.factory,this,widget,xpos,ypos,width,height);
 			}
 		}
 	}
-	cerr << "ObtainThumbnail - Release" << endl;
-
 
 	gdk_draw_pixbuf(widget->window,NULL,thumbnail,
 		fit->xoffset,fit->yoffset,
@@ -552,13 +585,6 @@ void Layout_ImageInfo::DrawThumbnail(GtkWidget *widget,int xpos,int ypos,int wid
 	if(transformed)
 		g_object_unref(transformed);
 
-//	cerr << "Waiting for sync" << endl;
-//	if(hrrenderthread)
-//		hrrenderthread->WaitSync();
-//	else
-//		cerr << "No thread" << endl;
-//	cerr << "Done" << endl;
-
 	delete fit;
 }
 
@@ -591,15 +617,13 @@ GdkPixbuf *Layout_ImageInfo::GetThumbnail()
 	if(layout.state.batchmode)
 		return(NULL);
 
-	cerr << "Getting thumbnail" << endl;
-
 	if(maskfilename && !mask)
 	{
 		mask=egg_pixbuf_get_thumbnail_for_file (maskfilename, EGG_PIXBUF_THUMBNAIL_LARGE, &err);
-//		cerr << "Attempting to load mask from: " << maskfilename << endl;
+//		Debug[TRACE] << "Attempting to load mask from: " << maskfilename << endl;
 		if(!mask)
 		{
-//			cerr << "Failed." << endl;
+			Debug[WARN] << "Mask loading failed - trying ImageSource method" << endl;
 			try
 			{
 				ImageSource *src=ISLoadImage(maskfilename);
@@ -620,24 +644,21 @@ GdkPixbuf *Layout_ImageInfo::GetThumbnail()
 			}
 			catch(const char *err)
 			{
-				cerr << "Error: " << err << endl;
+				Debug[ERROR] << "Error: " << err << endl;
 			}	
 			if(!mask)
 			{
 				if(err && err->message)
-					cerr << "Error: " << err->message << endl;
+					Debug[ERROR] << "Error: " << err->message << endl;
 				else
-					cerr << "Can't get mask thumbnail" << endl;
+					Debug[ERROR] << "Can't get mask thumbnail" << endl;
 				free(maskfilename);
 				maskfilename=NULL;
 			}
 		}
 	}
 
-//	if(thumbnail)
-//		return(thumbnail);
-
-	cerr << "Thumbnail not cached - loading..." << endl;
+	Debug[TRACE] << "Thumbnail not cached - loading..." << endl;
 
 	ImageSource *src=NULL;
 		
@@ -645,7 +666,7 @@ GdkPixbuf *Layout_ImageInfo::GetThumbnail()
 
 	if(!thumbnail)
 	{
-		cerr << "Can't get pixbuf - loading thumbnail via ImageSource..." << endl;
+		Debug[WARN] << "Can't get pixbuf - loading thumbnail via ImageSource..." << endl;
 		src=ISLoadImage(filename);
 		if(src)
 		{
@@ -685,11 +706,10 @@ GdkPixbuf *Layout_ImageInfo::GetThumbnail()
 
 
 	// If there's no display profile, then we can use the Default RGB profile instead...
-//	cerr << "Checking for Display Profile..." << endl;
+//	Debug[TRACE] << "Checking for Display Profile..." << endl;
 	CMSProfile *targetprof;
 	CMColourDevice target=CM_COLOURDEVICE_NONE;
 	if((targetprof=layout.state.profilemanager.GetProfile(CM_COLOURDEVICE_PRINTERPROOF)))
-//		target=CM_COLOURDEVICE_DISPLAY;
 		target=CM_COLOURDEVICE_PRINTERPROOF;
 	else if((targetprof=layout.state.profilemanager.GetProfile(CM_COLOURDEVICE_DISPLAY)))
 		target=CM_COLOURDEVICE_DISPLAY;
@@ -700,7 +720,6 @@ GdkPixbuf *Layout_ImageInfo::GetThumbnail()
 
 	if(target!=CM_COLOURDEVICE_NONE)
 	{
-		cerr << "Found - loading image to check for embedded profile..." << endl;
 		if(!src)
 		{
 			src=ISLoadImage(filename);
@@ -716,11 +735,11 @@ GdkPixbuf *Layout_ImageInfo::GetThumbnail()
 				emb=src->GetEmbeddedProfile();
 			if(emb)
 			{
-//				cerr << "Creating embedded->monitor transform..." << endl;
+//				Debug[TRACE] << "Creating embedded->monitor transform..." << endl;
 				if(emb->GetColourSpace()!=IS_TYPE_RGB)
 				{
 //					Need to replace the RGB thumbnail with a CMYK or Greyscale version!
-					cerr << "Creating new thumbnail - CMYK->monitor" << endl;
+					Debug[TRACE] << "Creating new thumbnail - CMYK->monitor" << endl;
 					int w,h;
 					w=(src->width*256)/src->height;
 					h=256;
@@ -744,13 +763,13 @@ GdkPixbuf *Layout_ImageInfo::GetThumbnail()
 			}
 			else
 			{
-//				cerr << "Creating default->monitor transform..." << endl;
+//				Debug[TRACE] << "Creating default->monitor transform..." << endl;
 				transform = layout.factory->GetTransform(target,IS_TYPE_RGB,customintent);
 			}
 		}
 		if(transform)
 		{
-//			cerr << "Applying transform..." << endl;
+//			Debug[TRACE] << "Applying transform..." << endl;
 			ImageSource *src2=new ImageSource_GdkPixbuf(thumbnail);
 			src2=new ImageSource_CMS(src2,transform);
 			GdkPixbuf *tn2=pixbuf_from_imagesource(src2);
@@ -765,7 +784,7 @@ GdkPixbuf *Layout_ImageInfo::GetThumbnail()
 		delete src;		
 	}
 
-	cerr << "done" << endl;
+	Debug[TRACE] << "done" << endl;
 
 	return(thumbnail);
 }
@@ -774,8 +793,6 @@ GdkPixbuf *Layout_ImageInfo::GetThumbnail()
 LayoutRectangle *Layout_ImageInfo::GetBounds()
 {
 	// Dummy function - override in subclasses!
-//	LayoutRectangle *result=new LayoutRectangle(0,0,100,100);
-//	return(result);
 	throw "Layout_ImageInfo::GetBounds() method should be overridden by subclass!";
 }
 
@@ -809,6 +826,15 @@ ImageSource *Layout_ImageInfo::GetImageSource(CMColourDevice target,CMTransformF
 	if(STRIP_ALPHA(is->type)==IS_TYPE_BW)
 		is=new ImageSource_Promote(is,colourspace);
 
+	// If this fails we don't bother with the histogram, since another thread has it
+	// locked for writing.
+	if(histogram.AttemptMutexShared())
+	{
+		is=new PPIS_Histogram(is,histogram);
+		histogram.ReleaseMutexShared();	// ReleaseShared because the Histogram itself holds an exclusive lock
+										// and we don't want to cancel its exclusivity!
+	}
+
 	CMSTransform *transform=NULL;
 
 	if(factory)
@@ -821,12 +847,12 @@ ImageSource *Layout_ImageInfo::GetImageSource(CMColourDevice target,CMTransformF
 		
 		if(emb)
 		{
-//				cerr << "Has embedded / assigned profile..." << endl;
+//				Debug[TRACE] << "Has embedded / assigned profile..." << endl;
 			transform=factory->GetTransform(target,emb,customintent);  // FIXME: intent!
 		}
 		else
 		{
-//				cerr << "No embedded profile - using default" << endl;
+//				Debug[TRACE] << "No embedded profile - using default" << endl;
 			transform=factory->GetTransform(target,IS_TYPE(STRIP_ALPHA(is->type)),customintent);
 		}
 
@@ -846,7 +872,7 @@ ImageSource *Layout_ImageInfo::ApplyMask(ImageSource *is)
 		if((is->width>is->height)^(mask->width>mask->height))
 		{
 			mask=new ImageSource_Rotate(mask,90);
-//			cerr << "Rotating mask" << endl;
+//			Debug[TRACE] << "Rotating mask" << endl;
 		}
 		mask=ISScaleImageBySize(mask,is->width,is->height,IS_SCALING_AUTOMATIC);
 		mask=new ImageSource_Invert(mask);
@@ -893,20 +919,9 @@ void Layout_ImageInfo::CancelRenderThread()
 {
 	if(hrrenderthread)
 	{
-		hrrenderthread->Stop();
-
-		// Obtain the mutex - this won't succeed until the thread has finished whatever it's doing...
-		// We defer this so that, for instance, all threads can be cancelled before we start waiting...
-//		while(!PPEffectHeader::AttemptMutex())
-//		{
-//			cerr << "Can't get exclusive lock - performing main loop iteration" << endl;
-//			gtk_main_iteration();
-//		}
-//		ReleaseMutex();
-
-//		while(!hrrenderthread->TestFinished())
-//			gtk_main_iteration();
-//		delete hrrenderthread;
+		hrrenderthread->Stop();	// We don't actually delete it here - the thread is responsible for its own
+								// demise (by way of a GTK Idle function running on the main thread's context)
+								// but having signalled it to stop, we can discard this pointer to it.
 	}
 	hrrenderthread=NULL;
 }
@@ -937,10 +952,6 @@ void Layout_ImageInfo::AssignProfile(const char *filename)
 	customprofile=NULL;
 	if(filename)
 		customprofile=strdup(filename);
-//	if(customprofile)
-//		cerr << "AssignProfile:  Custom Profile now set to " << customprofile << endl;
-//	else
-//		cerr << "AssignProfile:  No custom profile set" << endl;
 	FlushThumbnail();
 }
 
@@ -988,16 +999,26 @@ int Layout_ImageInfo::GetYRes()
 }
 
 
+// Because the only reason you would need to ObtainMutex() the ImageInfo (rather than ObtainShared())
+// is to make a write-change to it, we flush the high-res preview on the assumption that the change
+// will invalidate it.  Note, also, we won't be able to obtain the exclusive lock while the thread's
+// running.
+// FIXME - would be better to require an explicit flush() of some kind.
 void Layout_ImageInfo::ObtainMutex()
 {
-	cerr << "In custom Obtain method - flushing preview..." << endl;
+//	Debug[TRACE] << "In custom Obtain method - flushing preview..." << endl;
 	FlushHRPreview();
-	cerr << "Now attempting to obtain exclusive lock..." << endl;
+//	Debug[TRACE] << "Now attempting to obtain exclusive lock..." << endl;
 	while(!PPEffectHeader::AttemptMutex())
 	{
-		cerr << "Can't get exclusive lock - performing main loop iteration" << endl;
+//		Debug[TRACE] << "Can't get exclusive lock - performing main loop iteration" << endl;
 		gtk_main_iteration();
 	}
-	cerr << "Done" << endl;
+//	Debug[TRACE] << "Done" << endl;
 }
 
+
+PPHistogram &Layout_ImageInfo::GetHistogram()
+{
+	return(histogram);
+}
