@@ -1,6 +1,6 @@
 /*
  * PhotoPrint
- * Copyright (c) 2004-2006 by Alastair M. Robinson
+ * Copyright (c) 2004-2009 by Alastair M. Robinson
  * Distributed under the terms of the GNU General Public License -
  * see the file named "COPYING" for more details.
  *
@@ -18,7 +18,7 @@
 #include "config.h"
 #include "gettext.h"
 
-
+#include "support/debug.h"
 #include "support/configdb.h"
 #include "photoprint_state.h"
 
@@ -31,6 +31,8 @@
 #include "profilemanager/profileselector.h"
 #include "profilemanager/intentselector.h"
 #include "miscwidgets/patheditor.h"
+
+#include "support/pathsupport.h"
 #include "support/util.h"
 
 
@@ -48,13 +50,14 @@ bool ParseOptions(int argc,char *argv[],char **presetname)
 		{"version",no_argument,NULL,'v'},
 		{"preset",required_argument,NULL,'p'},
 		{"batch",no_argument,NULL,'b'},
+		{"debug",required_argument,NULL,'d'},
 		{0, 0, 0, 0}
 	};
 
 	while(1)
 	{
 		int c;
-		c = getopt_long(argc,argv,"hvp:b",long_options,NULL);
+		c = getopt_long(argc,argv,"hvp:bd:",long_options,NULL);
 		if(c==-1)
 			break;
 		switch (c)
@@ -65,6 +68,7 @@ bool ParseOptions(int argc,char *argv[],char **presetname)
 				printf("\t -v --version\t\tdisplay version\n");
 				printf("\t -p --preset\t\tread a specific preset file\n");
 				printf("\t -b --batch\t\trun without user interface\n");
+				printf("\t -d --debug\t\tset debugging level - 0 for silent, 4 for verbose");
 				throw 0;
 				break;
 			case 'v':
@@ -77,6 +81,9 @@ bool ParseOptions(int argc,char *argv[],char **presetname)
 			case 'b':
 				batchmode=true;
 				break;
+			case 'd':
+				Debug.SetLevel(DebugLevel(atoi(optarg)));
+				break;
 		}
 	}
 	return(batchmode);
@@ -92,12 +99,16 @@ static void destroy( GtkWidget *widget,
 
 int main(int argc,char **argv)
 {
-	cerr << "Photoprint starting..." << endl;
+	Debug[TRACE] << "Photoprint starting..." << endl;
 	gboolean have_gtk=false;
 	char *presetname=NULL;
 
-	stp_init();
-	cerr << "Initialized Gutenprint" << endl;
+	Debug.SetLevel(WARN);
+#ifdef WIN32
+	char *logname=substitute_homedir("$HOME" SEARCHPATH_SEPARATOR_S ".photoprint_errorlog");
+	Debug.SetLogFile(logname);
+	delete logname;
+#endif
 
 	bool batchmode=ParseOptions(argc,argv,&presetname);
 	if(!batchmode)
@@ -108,7 +119,6 @@ int main(int argc,char **argv)
 	else
 		setlocale(LC_ALL,"");
 
-	cerr << "Setting up gettext... " << PACKAGE << ", " << LOCALEDIR << endl;
 	bindtextdomain(PACKAGE,LOCALEDIR);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 	textdomain(PACKAGE);
@@ -124,11 +134,8 @@ int main(int argc,char **argv)
 
 		PhotoPrint_State state(batchmode);
 
-		cerr << "State created..." << endl;
-
 		if(presetname)
 			state.SetFilename(presetname);
-		cerr << "Set initial filename" << endl;
 
 		if(have_gtk)
 			splash->SetMessage(_("Checking .photoprint directory..."));
@@ -145,24 +152,20 @@ int main(int argc,char **argv)
 			delete splash;
 		}
 
-		cerr << "Translation test: " << _("Initializing...") << endl;
-
 		state.NewLayout();
 
-		cerr << "Translation test: " << _("Initializing...") << endl;
-
 		if(batchmode)
 		{
 			try
 			{
-				cerr << "Running in batch mode" << endl;
+				Debug[TRACE] << "Running in batch mode" << endl;
 				if(argc>optind)
 				{
 					bool allowcropping=state.layoutdb.FindInt("AllowCropping");
 					enum PP_ROTATION rotation=PP_ROTATION(state.layoutdb.FindInt("Rotation"));
 					for(int i=optind;i<argc;++i)
 					{
-						cerr << "Adding file: " << argv[i] << endl;
+						Debug[TRACE] << "Adding file: " << argv[i] << endl;
 						state.layout->AddImage(argv[i],allowcropping,rotation);
 					}
 					ProgressText p;
@@ -171,7 +174,7 @@ int main(int argc,char **argv)
 			}
 			catch(const char *err)
 			{
-				cerr << "Error: " << err << endl;
+				Debug[ERROR] << "Error: " << err << endl;
 			}
 		}
 		else
@@ -179,14 +182,11 @@ int main(int argc,char **argv)
 			try
 			{
 				GtkWidget *mainwindow;
-		cerr << "Translation test: " << _("Initializing...") << endl;
 				mainwindow = pp_mainwindow_new(&state);
 				g_signal_connect (G_OBJECT (mainwindow), "destroy",
 					    G_CALLBACK (destroy), NULL);
 				gtk_widget_show (mainwindow);
 
-		cerr << "Translation test: " << _("Initializing...") << endl;
-		
 				if(argc>optind)
 				{
 					bool allowcropping=state.layoutdb.FindInt("AllowCropping");
@@ -202,12 +202,8 @@ int main(int argc,char **argv)
 					state.layout->SetCurrentPage(lastpage);
 				}
 	
-		cerr << "Translation test: " << _("Initializing...") << endl;
-
 				pp_mainwindow_refresh(PP_MAINWINDOW(mainwindow));
 
-		cerr << "Translation test: " << _("Initializing...") << endl;
-	
 				gtk_main ();
 			}
 			catch (const char *err)
@@ -218,7 +214,9 @@ int main(int argc,char **argv)
 	}
 	catch(const char *err)
 	{
-		cerr << "Error: " << err << endl;
+		if(have_gtk)
+			ErrorMessage_Dialog(err);
+		Debug[ERROR] << "Error: " << err << endl;
 	}
 	catch(int retcode)
 	{
