@@ -6,10 +6,14 @@
 #include <gtk/gtklabel.h>
 
 #include "layout.h"
+#include "photoprint_state.h"
+
+#include "support/debug.h"
 
 #include "effects/ppeffect.h"
 #include "effects/effectselector.h"
 #include "pp_imageinfo.h"
+#include "pp_histogram.h"
 
 #include "pp_imagecontrol.h"
 
@@ -31,12 +35,13 @@ static void pp_imagecontrol_init (pp_ImageControl *stpuicombo);
 static void effectselector_changed(GtkWidget *wid,gpointer *ob)
 {
 	pp_ImageControl *ic=(pp_ImageControl *)ob;
-	Layout_ImageInfo *ii=ic->layout->FirstSelected();
+	LayoutIterator it(*ic->layout);
+	Layout_ImageInfo *ii=it.FirstSelected();
 	while(ii)
 	{
 		effectselector_apply(EFFECTSELECTOR(ic->effectselector),ii);
 		ii->FlushThumbnail();
-		ii=ic->layout->NextSelected();
+		ii=it.NextSelected();
 	}
 	g_signal_emit(G_OBJECT (ic),pp_imagecontrol_signals[CHANGED_SIGNAL], 0);
 }
@@ -44,13 +49,14 @@ static void effectselector_changed(GtkWidget *wid,gpointer *ob)
 
 static void effectselector_addeffect(GtkWidget *wid,gpointer *ob)
 {
-	cerr << "Acting on addeffect signal" << endl;
+	Debug[TRACE] << "Acting on addeffect signal" << endl;
 	pp_ImageControl *ic=(pp_ImageControl *)ob;
-	Layout_ImageInfo *ii=ic->layout->FirstSelected();
+	LayoutIterator it(*ic->layout);
+	Layout_ImageInfo *ii=it.FirstSelected();
 	while(ii)
 	{
 		effectselector_add_selected_effect(EFFECTSELECTOR(ic->effectselector),ii);
-		ii=ic->layout->NextSelected();
+		ii=it.NextSelected();
 	}
 	g_signal_emit(G_OBJECT (ic),pp_imagecontrol_signals[CHANGED_SIGNAL], 0);
 }
@@ -58,13 +64,14 @@ static void effectselector_addeffect(GtkWidget *wid,gpointer *ob)
 
 static void effectselector_removeeffect(GtkWidget *wid,gpointer *ob)
 {
-	cerr << "Acting on removeeffect signal" << endl;
+	Debug[TRACE] << "Acting on removeeffect signal" << endl;
 	pp_ImageControl *ic=(pp_ImageControl *)ob;
-	Layout_ImageInfo *ii=ic->layout->FirstSelected();
+	LayoutIterator it(*ic->layout);
+	Layout_ImageInfo *ii=it.FirstSelected();
 	while(ii)
 	{
 		effectselector_remove_selected_effect(EFFECTSELECTOR(ic->effectselector),ii);
-		ii=ic->layout->NextSelected();
+		ii=it.NextSelected();
 	}
 	g_signal_emit(G_OBJECT (ic),pp_imagecontrol_signals[CHANGED_SIGNAL], 0);
 }
@@ -72,16 +79,29 @@ static void effectselector_removeeffect(GtkWidget *wid,gpointer *ob)
 
 void pp_imagecontrol_refresh(pp_ImageControl *ob)
 {
+	Debug[TRACE] << "pp_imagecontrol_refresh: refreshing imageinfo" << endl;
 	pp_imageinfo_refresh(PP_IMAGEINFO(ob->imageinfo));
+	Debug[TRACE] << "pp_imagecontrol_refresh: refreshing histogram" << endl;
+	pp_histogram_refresh(PP_HISTOGRAM(ob->histogram));
+	Debug[TRACE] << "pp_imagecontrol_refresh: done" << endl;
 }
 
 
 void pp_imagecontrol_set_image(pp_ImageControl *ob)
 {
-	Layout_ImageInfo *ii=ob->layout->FirstSelected();
+	LayoutIterator it(*ob->layout);
+	Layout_ImageInfo *ii=it.FirstSelected();
 	pp_imageinfo_change_image(PP_IMAGEINFO(ob->imageinfo));
 	effectselector_set_current_list(EFFECTSELECTOR(ob->effectselector),ii);
 	pp_imageinfo_refresh(PP_IMAGEINFO(ob->imageinfo));
+	pp_histogram_refresh(PP_HISTOGRAM(ob->histogram));
+}
+
+
+static void expander_callback (GObject *object, GParamSpec *param_spec, gpointer userdata)
+{
+	pp_ImageControl *ob=PP_IMAGECONTROL(userdata);
+	ob->layout->state.SetInt("ExpanderState_EffectSelector",gtk_expander_get_expanded (GTK_EXPANDER(ob->expander1)));
 }
 
 
@@ -99,6 +119,9 @@ pp_imagecontrol_new (Layout *layout)
 	gtk_box_pack_start(GTK_BOX(ob),ob->imageinfo,FALSE,FALSE,0);
 	gtk_widget_show(ob->imageinfo);
 
+	ob->histogram=pp_histogram_new(layout);
+	gtk_box_pack_start(GTK_BOX(ob),ob->histogram,FALSE,FALSE,0);
+	gtk_widget_show(ob->histogram);
 
 	// FIXME add transformations here
 
@@ -109,7 +132,8 @@ pp_imagecontrol_new (Layout *layout)
 	// EffectSelector
 
 	ob->expander1=frame=gtk_expander_new(_("Effects"));
-	gtk_expander_set_expanded(GTK_EXPANDER(frame),true);
+	g_signal_connect(ob->expander1, "notify::expanded",G_CALLBACK (expander_callback), ob);
+	gtk_expander_set_expanded(GTK_EXPANDER(frame),layout->state.FindInt("ExpanderState_EffectSelector"));
 
 	gtk_box_pack_start(GTK_BOX(ob),frame,TRUE,TRUE,0);
 	gtk_widget_show(frame);
@@ -172,24 +196,3 @@ pp_imagecontrol_init (pp_ImageControl *ob)
 	ob->expander1=NULL;
 }
 
-
-#define IMAGEINFO_MASK 0x1
-#define EXPANDER1_MASK 0x2
-
-int pp_imagecontrol_get_expander_state(pp_ImageControl *ob)
-{
-	int result=0;
-	if(gtk_expander_get_expanded(GTK_EXPANDER(ob->imageinfo)))
-		result|=IMAGEINFO_MASK;
-	if(gtk_expander_get_expanded(GTK_EXPANDER(ob->expander2)))
-		result|=EXPANDER1_MASK;
-
-	return(result);
-}
-
-
-void pp_imagecontrol_set_expander_state(pp_ImageControl *ob,int state)
-{
-	gtk_expander_set_expanded(GTK_EXPANDER(ob->imageinfo),(state&IMAGEINFO_MASK)!=0);
-	gtk_expander_set_expanded(GTK_EXPANDER(ob->expander1),(state&EXPANDER1_MASK)!=0);
-}
