@@ -6,6 +6,7 @@
 #include <gtk/gtkdnd.h>
 #include <gtk/gtkmenu.h>
 
+#include "support/debug.h"
 #include "support/layoutrectangle.h"
 #include "stpui_widgets/stpui_combo.h"
 #include "support/progressbar.h"
@@ -72,7 +73,7 @@ static void get_dnd_data(GtkWidget *widget, GdkDragContext *context,
 		}
 		else
 		{	
-			cerr << "URIList: " << urilist << endl;
+			Debug[TRACE] << "URIList: " << urilist << endl;
 			gchar *uri=urilist;
 			while(*urilist && *urilist!='\n' && *urilist!='\r')
 				++urilist;
@@ -308,7 +309,8 @@ pp_layout_carousel_pageview_expose( GtkWidget      *widget,
 
 	pageview->layout->DrawPreview(widget,pageview->left,pageview->top,pageview->width,pageview->height);
 
-	Layout_ImageInfo *ii=pageview->layout->FirstSelected();
+	LayoutIterator it(*pageview->layout);
+	Layout_ImageInfo *ii=it.FirstSelected();
 	while(ii)
 	{
 		if(ii->page==pageview->layout->GetCurrentPage())
@@ -324,7 +326,7 @@ pp_layout_carousel_pageview_expose( GtkWidget      *widget,
 				pageview->left+s->x-1,pageview->top+s->y-1,s->w+2,s->h+2);
 			delete s;
 		}
-		ii=pageview->layout->NextSelected();
+		ii=it.NextSelected();
 	}
 
 
@@ -381,7 +383,8 @@ pp_layout_carousel_pageview_button_press( GtkWidget      *widget,
 					{
 						if(prevselected)
 						{
-							Layout_ImageInfo *ii=pageview->layout->FirstImage();
+							LayoutIterator it(*pageview->layout);
+							Layout_ImageInfo *ii=it.FirstImage();
 							bool selecting=false;
 							while(ii)
 							{
@@ -393,7 +396,7 @@ pp_layout_carousel_pageview_button_press( GtkWidget      *widget,
 								}
 								if(selecting)
 									ii->SetSelected(true);
-								ii=pageview->layout->NextImage();
+								ii=it.NextImage();
 							}
 						}
 						else
@@ -499,6 +502,7 @@ pp_layout_carousel_pageview_button_release( GtkWidget      *widget,
 
 void pp_layout_carousel_pageview_refresh(pp_Layout_Carousel_PageView *pv)
 {
+	pv->layout->FlushPreview();
 	gtk_widget_queue_draw (GTK_WIDGET (pv));
 }
 
