@@ -7,6 +7,8 @@
 
 #include "stpui_widgets/stpui_combo.h"
 #include "pp_sigcontrol.h"
+
+#include "support/debug.h"
 #include "support/progressbar.h"
 
 #include "pp_menu_image.h"
@@ -74,7 +76,7 @@ static void get_dnd_data(GtkWidget *widget, GdkDragContext *context,
 		}
 		else
 		{	
-			cerr << "URIList: " << urilist << endl;
+			Debug[TRACE] << "URIList: " << urilist << endl;
 			gchar *uri=urilist;
 			while(*urilist && *urilist!='\n' && *urilist!='\r')
 				++urilist;
@@ -388,7 +390,8 @@ pp_layout_nup_pageview_button_press( GtkWidget      *widget,
 					{
 						if(prevselected)
 						{
-							Layout_ImageInfo *ii=pageview->layout->FirstImage();
+							LayoutIterator it(*pageview->layout);
+							Layout_ImageInfo *ii=it.FirstImage();
 							bool selecting=false;
 							while(ii)
 							{
@@ -400,7 +403,7 @@ pp_layout_nup_pageview_button_press( GtkWidget      *widget,
 								}
 								if(selecting)
 									ii->SetSelected(true);
-								ii=pageview->layout->NextImage();
+								ii=it.NextImage();
 							}
 						}
 						else
