@@ -7,6 +7,7 @@
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkexpander.h>
 
+#include "support/debug.h"
 #include "stpui_widgets/stpui_combo.h"
 #include "stpui_widgets/dimension.h"
 #include "layout.h"
@@ -116,7 +117,7 @@ static void vtiles_changed(GtkWidget *wid,gpointer *ob)
 	Layout_Poster *l=(Layout_Poster*)lo->state->layout;
 	l->FlushHRPreviews();
 
-	cerr << "In VTiles_Changed" << endl;
+	Debug[TRACE] << "In VTiles_Changed" << endl;
 
 	l->vtiles=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lo->vtiles));
 	l->SizeFromTiles();
@@ -175,7 +176,7 @@ static void pageview_changed(GtkWidget *wid,gpointer *ob)
 static void pageview_popupmenu(GtkWidget *wid,gpointer *ob)
 {
 	pp_Layout_Poster *lo=(pp_Layout_Poster *)ob;
-	cerr << "Forwarding popupmenu signal..." << endl;
+	Debug[TRACE] << "Forwarding popupmenu signal..." << endl;
 	g_signal_emit(G_OBJECT (lo),pp_layout_poster_signals[POPUPMENU_SIGNAL], 0);
 }
 
@@ -248,8 +249,21 @@ void pp_layout_poster_set_unit(GtkWidget *wid,enum Units unit)
 }
 
 
-GtkWidget*
-pp_layout_poster_new (PhotoPrint_State *state)
+static void expander_callback (GObject *object, GParamSpec *param_spec, gpointer userdata)
+{
+	pp_Layout_Poster *ob=PP_LAYOUT_POSTER(userdata);
+	ob->state->SetInt("ExpanderState_Poster",gtk_expander_get_expanded (GTK_EXPANDER(object)));
+}
+
+
+static void expander_callback2 (GObject *object, GParamSpec *param_spec, gpointer userdata)
+{
+	pp_Layout_Poster *ob=PP_LAYOUT_POSTER(userdata);
+	ob->state->SetInt("ExpanderState_PosterOverlap",gtk_expander_get_expanded (GTK_EXPANDER(object)));
+}
+
+
+GtkWidget *pp_layout_poster_new (PhotoPrint_State *state)
 {
 	pp_Layout_Poster *ob=PP_LAYOUT_POSTER(g_object_new (pp_layout_poster_get_type (), NULL));
 
@@ -318,25 +332,11 @@ pp_layout_poster_new (PhotoPrint_State *state)
 //	gtk_container_foreach(GTK_CONTAINER(scrollwin),killshadow,NULL);
 	gtk_widget_show (vbox);
 
-#if 0
-
-	GtkWidget *scrollwin=gtk_scrolled_window_new(NULL,NULL);
-    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
-                                    GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	GtkWidget *hbox2=gtk_hbox_new(FALSE,0);
-	gtk_widget_show (hbox2);
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox2),vbox,TRUE,TRUE,5);
-	
-	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrollwin), hbox2);
-	gtk_box_pack_start(GTK_BOX(&ob->hbox),scrollwin,FALSE,FALSE,0);
-	gtk_widget_show (vbox);
-	gtk_widget_show (scrollwin);
-#endif
 
 	// PosterSize
 	frame=gtk_expander_new(_("Poster Layout"));
-	gtk_expander_set_expanded(GTK_EXPANDER(frame),true);
+	gtk_expander_set_expanded(GTK_EXPANDER(frame),state->FindInt("ExpanderState_Poster"));
+	g_signal_connect(frame, "notify::expanded",G_CALLBACK (expander_callback), ob);
 	gtk_box_pack_start(GTK_BOX(vbox),frame,FALSE,FALSE,0);
 	gtk_widget_show(frame);
 
@@ -396,7 +396,8 @@ pp_layout_poster_new (PhotoPrint_State *state)
 	gtk_widget_show(ob->vtiles);
 
 	frame=gtk_expander_new(_("Overlap"));
-	gtk_expander_set_expanded(GTK_EXPANDER(frame),true);
+	gtk_expander_set_expanded(GTK_EXPANDER(frame),state->FindInt("ExpanderState_PosterOverlap"));
+	g_signal_connect(frame, "notify::expanded",G_CALLBACK (expander_callback2), ob);
 	gtk_box_pack_start(GTK_BOX(vbox),frame,FALSE,FALSE,0);
 	gtk_widget_show(frame);
 
