@@ -26,6 +26,7 @@
 #include "pp_menu_layout.h"
 #include "pp_menu_image.h"
 #include "pp_menu_options.h"
+#include "pp_menu_shortcuts.h"
 #include "pp_menu_help.h"
 
 #include "config.h"
@@ -57,7 +58,9 @@ static void layout_selection_changed(GtkWidget *wid,gpointer *ob)
 	PP_ROTATION rotation=ImageMenu_GetRotation(mw->uim);
 	bool allowcropping=ImageMenu_GetCropFlag(mw->uim);
 	bool removeimage=true;
-	Layout_ImageInfo *ii=mw->state->layout->FirstSelected();
+
+	LayoutIterator it(*mw->state->layout);
+	Layout_ImageInfo *ii=it.FirstSelected();
 	if(ii)
 	{
 		allowcropping=ii->allowcropping;
@@ -71,7 +74,7 @@ static void layout_selection_changed(GtkWidget *wid,gpointer *ob)
 		if(rotation!=ii->rotation)
 			rotation=PP_ROTATION_NONE;
 		allowcropping&=ii->allowcropping;
-		ii=mw->state->layout->NextSelected();
+		ii=it.NextSelected();
 	}
 
 	ImageMenu_SetCropFlag(mw->uim,allowcropping);
@@ -130,14 +133,13 @@ pp_mainwindow_new (PhotoPrint_State *state)
 	BuildLayoutMenu(ob,ob->uim);
 	BuildImageMenu(ob,ob->uim);
 	BuildOptionsMenu(ob,ob->uim);
+	BuildShortcutsMenu(ob,ob->uim);
 	BuildHelpMenu(ob,ob->uim);
 	
 	accel_group = gtk_ui_manager_get_accel_group (ob->uim);
 	gtk_window_add_accel_group (GTK_WINDOW (ob), accel_group);
 	
 	GtkWidget *tmp;
-	tmp=gtk_ui_manager_get_widget(ob->uim,"/MainMenu/FileMenu/PrintPreview");
-	gtk_widget_set_sensitive(tmp,false);
 	tmp=gtk_ui_manager_get_widget(ob->uim,"/MainMenu/EditMenu/Cut");
 	gtk_widget_set_sensitive(tmp,false);
 	tmp=gtk_ui_manager_get_widget(ob->uim,"/MainMenu/EditMenu/Copy");
