@@ -18,6 +18,8 @@
 #include <iostream>
 #include <fstream>
 
+#include "../support/debug.h"
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -41,6 +43,9 @@ CMSRGBPrimaries CMSPrimaries_CIE(0.7355,0.2645,0.2658,0.7243,0.1669,0.0085);
 
 CMSProfile::CMSProfile(const char *fn) : md5(NULL), generated(false), filename(NULL), buffer(NULL), buflen(0)
 {
+	if(!fn)
+		throw "NULL profile filename provided";
+
 	cmsErrorAction(LCMS_ERROR_SHOW);
 
 	filename=strdup(fn);
@@ -56,17 +61,17 @@ void CMSProfile::CalcMD5()
 {
 	if(generated)
 	{
-		cerr << "Saving profile to RAM for MD5 calculation." << endl;
+		Debug[TRACE] << "Saving profile to RAM for MD5 calculation." << endl;
 		size_t plen=0;
 		_cmsSaveProfileToMem(prof,NULL,&plen);
 		if(plen>0)
 		{
-			cerr << "Plen = " << plen << endl;
+			Debug[TRACE] << "Plen = " << plen << endl;
 			buflen=plen;
 			buffer=(char *)malloc(buflen);
 			if(_cmsSaveProfileToMem(prof,buffer,&plen))
 			{
-				cerr << "Saved successfully" << endl;
+				Debug[TRACE] << "Saved successfully" << endl;
 				md5=new MD5Digest(buffer+sizeof(icHeader),buflen-sizeof(icHeader));
 			}
 		}
@@ -91,7 +96,8 @@ void CMSProfile::CalcMD5()
 }
 
 
-CMSProfile::CMSProfile(CMSRGBPrimaries &primaries,CMSRGBGamma &gamma,CMSWhitePoint &whitepoint) : md5(NULL), generated(true), filename(NULL)
+CMSProfile::CMSProfile(CMSRGBPrimaries &primaries,CMSRGBGamma &gamma,CMSWhitePoint &whitepoint)
+	: md5(NULL), generated(true), filename(NULL), buffer(NULL), buflen(0)
 {
 	if(!(prof=cmsCreateRGBProfile(&whitepoint.whitepoint,&primaries,gamma.gammatables)))
 		throw "Can't create virtual RGB profile";
@@ -99,7 +105,8 @@ CMSProfile::CMSProfile(CMSRGBPrimaries &primaries,CMSRGBGamma &gamma,CMSWhitePoi
 }
 
 
-CMSProfile::CMSProfile(CMSGamma &gamma,CMSWhitePoint &whitepoint) : md5(NULL), generated(true), filename(NULL)
+CMSProfile::CMSProfile(CMSGamma &gamma,CMSWhitePoint &whitepoint)
+	: md5(NULL), generated(true), filename(NULL), buffer(NULL), buflen(0)
 {
 	if(!(prof=cmsCreateGrayProfile(&whitepoint.whitepoint,gamma.GetGammaTable())))
 		throw "Can't create virtual Grey profile";
@@ -107,7 +114,8 @@ CMSProfile::CMSProfile(CMSGamma &gamma,CMSWhitePoint &whitepoint) : md5(NULL), g
 }
 
 
-CMSProfile::CMSProfile(CMSWhitePoint &whitepoint) : md5(NULL), generated(true), filename(NULL)
+CMSProfile::CMSProfile(CMSWhitePoint &whitepoint)
+	: md5(NULL), generated(true), filename(NULL), buffer(NULL), buflen(0)
 {
 	if(!(prof=cmsCreateLabProfile(&whitepoint.whitepoint)))
 		throw "Can't create virtual LAB profile";
@@ -115,7 +123,8 @@ CMSProfile::CMSProfile(CMSWhitePoint &whitepoint) : md5(NULL), generated(true),
 }
 
 
-CMSProfile::CMSProfile(char *srcbuf,int length) : md5(NULL), generated(false), filename(NULL)
+CMSProfile::CMSProfile(char *srcbuf,int length)
+	: md5(NULL), generated(false), filename(NULL), buffer(NULL), buflen(0)
 {
 	buffer=(char *)malloc(length);
 	buflen=length;
@@ -126,7 +135,8 @@ CMSProfile::CMSProfile(char *srcbuf,int length) : md5(NULL), generated(false), f
 }
 
 
-CMSProfile::CMSProfile() : md5(NULL), generated(true), filename(NULL)
+CMSProfile::CMSProfile()
+	: md5(NULL), generated(true), filename(NULL), buffer(NULL), buflen(0)
 {
 	if(!(prof=cmsCreate_sRGBProfile()))
 		throw "Can't create virtual sRGB profile";
@@ -134,9 +144,10 @@ CMSProfile::CMSProfile() : md5(NULL), generated(true), filename(NULL)
 }
 
 
-CMSProfile::CMSProfile(const CMSProfile &src) : md5(NULL), generated(src.generated), filename(NULL)
+CMSProfile::CMSProfile(const CMSProfile &src)
+	: md5(NULL), generated(src.generated), filename(NULL), buffer(NULL), buflen(0)
 {
-	cerr << "In CMSProfile Copy Constructor" << endl;
+	Debug[TRACE] << "In CMSProfile Copy Constructor" << endl;
 	if(src.filename)
 	{
 		filename=strdup(src.filename);
@@ -188,7 +199,7 @@ bool CMSProfile::IsDeviceLink()
 
 bool CMSProfile::IsV4()
 {
-	cerr << "Profile version: " << cmsGetProfileICCversion(prof) << endl;
+	Debug[TRACE] << "Profile version: " << cmsGetProfileICCversion(prof) << endl;
 	return(cmsGetProfileICCversion(prof) >= 0x04000000L);
 }
 
@@ -335,7 +346,7 @@ bool CMSProfile::Save(const char *outfn)
 			}
 			if(outfn)
 			{
-				cerr << "Saving buffer: " << long(buffer) << ", length: " << buflen << endl;
+				Debug[TRACE] << "Saving buffer: " << long(buffer) << ", length: " << buflen << endl;
 				ofstream f(outfn,ios::out|ios::binary);
 				f.write(buffer,buflen);
 				f.close();
@@ -637,20 +648,20 @@ CMSProofingTransform::CMSProofingTransform(CMSProfile *in,CMSProfile *out,CMSPro
 	}
 
 //	if(in->GetFilename())
-//		cerr << "In: " << in->GetFilename() << endl;
-//	cerr << "In: " << in->GetDescription() << endl;
+//		Debug[TRACE] << "In: " << in->GetFilename() << endl;
+//	Debug[TRACE] << "In: " << in->GetDescription() << endl;
 	if(out)
 	{
 //		if(out->GetFilename())
-//			cerr << "Out: " << out->GetFilename() << endl;
-//		cerr << "Out: " << out->GetDescription() << endl;
+//			Debug[TRACE] << "Out: " << out->GetFilename() << endl;
+//		Debug[TRACE] << "Out: " << out->GetDescription() << endl;
 	}
 //	if(proof->GetFilename())
-//		cerr << "Proof: " << proof->GetFilename() << endl;
-//	cerr << "Proof: " << proof->GetDescription() << endl;
+//		Debug[TRACE] << "Proof: " << proof->GetFilename() << endl;
+//	Debug[TRACE] << "Proof: " << proof->GetDescription() << endl;
 
-//	cerr << "Viewing intent: " << viewintent << endl;
-//	cerr << "Rendering intent: " << proofintent << endl;
+//	Debug[TRACE] << "Viewing intent: " << viewintent << endl;
+//	Debug[TRACE] << "Rendering intent: " << proofintent << endl;
 
 	transform = cmsCreateProofingTransform(in->prof,
 		it,
@@ -701,14 +712,14 @@ CMSProofingTransform::CMSProofingTransform(CMSProfile *devicelink,CMSProfile *pr
 	}
 
 //	if(devicelink->GetFilename())
-//		cerr << "DeviceLink: " << devicelink->GetFilename() << endl;
-//	cerr << "DeviceLink: " << devicelink->GetDescription() << endl;
+//		Debug[TRACE] << "DeviceLink: " << devicelink->GetFilename() << endl;
+//	Debug[TRACE] << "DeviceLink: " << devicelink->GetDescription() << endl;
 //	if(proof->GetFilename())
-//		cerr << "Proof: " << proof->GetFilename() << endl;
-//	cerr << "Proof: " << proof->GetDescription() << endl;
+//		Debug[TRACE] << "Proof: " << proof->GetFilename() << endl;
+//	Debug[TRACE] << "Proof: " << proof->GetDescription() << endl;
 
-//	cerr << "Viewing intent: " << viewintent << endl;
-//	cerr << "Rendering intent: " << proofintent << endl;
+//	Debug[TRACE] << "Viewing intent: " << viewintent << endl;
+//	Debug[TRACE] << "Rendering intent: " << proofintent << endl;
 
 	transform = cmsCreateProofingTransform(devicelink->prof,
 		it,
