@@ -14,11 +14,6 @@
 
 #include "units.h"
 
-#include "../config.h"
-#include "../gettext.h"
-#define _(x) gettext(x)
-#define N_(x) gettext_noop(x)
-
 G_BEGIN_DECLS
 
 #define DIMENSION_TYPE			(dimension_get_type())
