@@ -1,6 +1,8 @@
 #ifndef SEARCHPATH_H
 #define SEARCHPATH_H
 
+#include <list>
+
 #include <sys/types.h>
 #include <dirent.h>
 #include <ostream>
@@ -24,8 +26,8 @@
 		
 	void AddPath(const char *path)
 		Adds a new search path to the list.  UNIX-style multiple paths are supported
-		with ':' as the separator character.  $HOME at the start of a path will be replaced
-		by the contents of the HOME environment variable.
+		with ':' as the separator character (Use ; on Windows).
+		$HOME at the start of a path will be replaced by the contents of the HOME environment variable.
 		
 	void RemovePath(const char *path);
 		Attempts to remove the specified path from the list.  Fails silently if
@@ -69,6 +71,22 @@
 
 
 class SearchPathInstance;
+class SearchPathHandler;
+
+class SearchPathIterator
+{
+	public:
+	SearchPathIterator(SearchPathHandler &header);
+	virtual ~SearchPathIterator();
+	virtual const char *GetNextFilename(const char *last);
+	const char *GetNextPath(const char *last);
+	protected:
+	SearchPathHandler &header;
+	std::list<SearchPathInstance *>::iterator spiterator;
+	DIR *searchdirectory;
+	char *searchfilename;
+};
+
 
 class SearchPathHandler
 {
@@ -80,17 +98,16 @@ class SearchPathHandler
 	virtual void AddPath(const char *path);
 	virtual void RemovePath(const char *path);
 	virtual void ClearPaths();
-	virtual const char *GetNextFilename(const char *last);
-	const char *GetNextPath(const char *last);
 	char *GetPaths();
 	protected:
 	SearchPathInstance *FindPath(const char *path);
-	SearchPathInstance *first;
+	std::list<SearchPathInstance *> paths;
 	// Used by GetNextFilename();
-	DIR *searchdirectory;
-	char *searchfilename;
-	SearchPathInstance *searchiterator;
+//	DIR *searchdirectory;
+//	char *searchfilename;
+	SearchPathIterator *searchiterator;
 	friend class SearchPathInstance;
+	friend class SearchPathIterator;
 	friend std::ostream& operator<<(std::ostream &s,SearchPathHandler &sp);
 };
 
